// Game variables
let canvas: HTMLCanvasElement;
let ctx: CanvasRenderingContext2D;
let scoreEl: HTMLElement | null;
let finalScoreEl: HTMLElement | null;
let gameOverEl: HTMLElement | null;
let gameLoop: number;
let score = 0;
let gameSpeed = 5;
let gameOver = false;
let gameStarted = false;
let highScore = parseInt(localStorage.getItem('runnerHighScore') || '0');

// Background elements
const background = {
    clouds: [] as Array<{
        x: number;
        y: number;
        width: number;
        height: number;
        speed: number;
        puffs: Array<{
            x: number;
            y: number;
            radius: number;
        }>;
    }>,
    mountains: [] as Array<{
        x: number;
        height: number;
    }>,
    stars: [] as Array<{
        x: number;
        y: number;
        size: number;
        twinkle: number;
    }>,
    time: 0
};

// Selected character tracking
export let selectedCharacter = 'classic';

// Character types
type BaseCharacter = {
    color: string;
    accentColor: string;
    eyeColor: string;
    shape: 'round' | 'diamond' | 'square' | 'oval';
    size: {
        width: number;
        height: number;
    };
    physics: {
        gravity: number;
        jumpForce: number;
    };
};

type ClassicCharacter = BaseCharacter & {
    shape: 'round';
    effects: {
        hasGlow: boolean;
        hasShine: boolean;
        trailColor: string;
        personality: {
            expression: string;
            emotes: string[];
            soundEffects: string[];
        };
    };
};

type NinjaCharacter = BaseCharacter & {
    shape: 'diamond';
    bandColor: string;
    effects: {
        hasShadowTrail: boolean;
        hasMetallicSheen: boolean;
        trailColor: string;
        personality: {
            expression: string;
            emotes: string[];
            soundEffects: string[];
        };
    };
};

type RobotCharacter = BaseCharacter & {
    shape: 'square';
    screenColor: string;
    effects: {
        hasMetallicPanels: boolean;
        hasScreenGlow: boolean;
        trailColor: string;
        metalSheen: {
            primary: string;
            secondary: string;
            highlight: string;
        };
        circuitPatterns: {
            primaryColor: string;
            secondaryColor: string;
            glowIntensity: number;
        };
        industrialDetails: {
            rivets: boolean;
            gearEffects: boolean;
            steamPuffs: boolean;
        };
        personality: {
            expression: string;
            emotes: string[];
            soundEffects: string[];
        };
    };
};

type AlienCharacter = BaseCharacter & {
    shape: 'oval';
    effects: {
        hasGlow: boolean;
        hasAntenna: boolean;
        hasThirdEye: boolean;
        trailColor: string;
        personality: {
            expression: string;
            emotes: string[];
            soundEffects: string[];
        };
    };
};

type CharacterConfig = {
    classic: ClassicCharacter;
    ninja: NinjaCharacter;
    robot: RobotCharacter;
    alien: AlienCharacter;
};

// Characters configuration
export const characters: CharacterConfig = {
    classic: {
        color: '#ff6b6b',
        accentColor: '#ff4757',
        eyeColor: '#fff',
        shape: 'round',
        size: {
            width: 40,
            height: 40
        },
        physics: {
            gravity: 0.7,
            jumpForce: -15
        },
        effects: {
            hasGlow: true,
            hasShine: true,
            trailColor: 'rgba(255, 107, 107, 0.3)',
            personality: {
                expression: 'determined',
                emotes: ['💪', '✨', '🔥'],
                soundEffects: ['whoosh', 'pow', 'zing']
            }
        }
    },
    ninja: {
        color: '#2d3436',
        accentColor: '#636e72',
        bandColor: '#e74c3c',
        eyeColor: '#fff',
        shape: 'diamond',
        size: {
            width: 35,
            height: 45
        },
        physics: {
            gravity: 0.6,
            jumpForce: -17
        },
        effects: {
            hasShadowTrail: true,
            hasMetallicSheen: true,
            trailColor: 'rgba(45, 52, 54, 0.3)',
            personality: {
                expression: 'focused',
                emotes: ['⚔️', '🌟', '💨'],
                soundEffects: ['swish', 'slice', 'dash']
            }
        }
    },
    robot: {
        color: '#ff9f43',
        accentColor: '#2e86de',
        screenColor: '#34495e',
        eyeColor: '#2e86de',
        shape: 'square',
        size: {
            width: 48,
            height: 52
        },
        physics: {
            gravity: 0.8,
            jumpForce: -16
        },
        effects: {
            hasMetallicPanels: true,
            hasScreenGlow: true,
            trailColor: 'rgba(255, 159, 67, 0.3)',
            metalSheen: {
                primary: '#ff9f43',
                secondary: '#2e86de',
                highlight: '#ecf0f1'
            },
            circuitPatterns: {
                primaryColor: '#f1c40f',
                secondaryColor: '#34495e',
                glowIntensity: 0.8
            },
            industrialDetails: {
                rivets: true,
                gearEffects: true,
                steamPuffs: true
            },
            personality: {
                expression: 'analytical',
                emotes: ['🤖', '⚡', '🔧'],
                soundEffects: ['beep', 'whir', 'zap']
            }
        }
    },
    alien: {
        color: '#00b894',
        accentColor: '#00cec9',
        eyeColor: '#fff',
        shape: 'oval',
        size: {
            width: 35,
            height: 35
        },
        physics: {
            gravity: 0.5,
            jumpForce: -16
        },
        effects: {
            hasGlow: true,
            hasAntenna: true,
            hasThirdEye: true,
            trailColor: 'rgba(0, 184, 148, 0.3)',
            personality: {
                expression: 'mysterious',
                emotes: ['👽', '🌌', '✨'],
                soundEffects: ['warp', 'pulse', 'beam']
            }
        }
    }
};

// Update player properties
const player = {
    x: 100,
    y: 0,
    width: 40,
    height: 40,
    velocityY: 0,
    jumping: false,
    doubleJumped: false,
    color: characters.classic.color,
    gravity: characters.classic.physics.gravity,
    jumpForce: characters.classic.physics.jumpForce,
    maxVelocityY: 15,
    squash: 1,
    rotation: 0,
    isTeleporting: false,
    teleportCooldown: 0,
    teleportDuration: 0,
    teleportParticles: [] as Particle[],
    phaseAlpha: 1,
    phaseDirection: 1,
    isParachuteActive: false,
    parachuteDeployed: false // Add this line
};

// Remove platforms array and keep only obstacles
let obstacles: (Platform | Bird)[] = [];

// Update game variables
const INITIAL_GAME_SPEED = 5;
const MAX_SPEED_INCREASE = 15;
const SPEED_INCREASE_INTERVAL = 10; // Score points between speed increases
const SPEED_INCREASE_AMOUNT = 0.3;  // Smaller increment for smoother acceleration
let frame = 0;

// Add difficulty scaling
function updateGameSpeed() {
    // Calculate target speed based on score
    const speedIncrease = Math.min(
        Math.floor(score / SPEED_INCREASE_INTERVAL) * SPEED_INCREASE_AMOUNT,
        MAX_SPEED_INCREASE
    );
    
    // Smooth transition to target speed
    const targetSpeed = INITIAL_GAME_SPEED + speedIncrease;
    gameSpeed += (targetSpeed - gameSpeed) * 0.1;
    
    // Ensure speed stays within bounds
    gameSpeed = Math.max(INITIAL_GAME_SPEED, Math.min(gameSpeed, INITIAL_GAME_SPEED + MAX_SPEED_INCREASE));
}

// Add obstacle types
const OBSTACLE_TYPES = {
    NORMAL: 'normal',
    TALL: 'tall',
    WIDE: 'wide',
    FLOATING: 'floating',
    SPIKY: 'spiky'
};

// Platform class
class Platform {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    color: string;
    accentColor: string;
    roundness: number;
    addSpikes: boolean = false;
    type: string;
    hoverOffset: number = 0;
    hoverSpeed: number = 0.05;

    constructor(x: number, y: number, width: number) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.speed = gameSpeed;
        this.roundness = 8;
        
        // Determine obstacle type based on score and randomness
        if (score > 30 && Math.random() < 0.2) {
            this.type = OBSTACLE_TYPES.FLOATING;
        } else if (score > 20 && Math.random() < 0.3) {
            this.type = OBSTACLE_TYPES.SPIKY;
        } else if (Math.random() < 0.3) {
            this.type = OBSTACLE_TYPES.TALL;
        } else if (Math.random() < 0.3) {
            this.type = OBSTACLE_TYPES.WIDE;
        } else {
            this.type = OBSTACLE_TYPES.NORMAL;
        }

        // Set dimensions based on type
        switch (this.type) {
            case OBSTACLE_TYPES.TALL:
                this.width = 30 + Math.random() * 20;
                this.height = 80 + Math.random() * 40;
                break;
            case OBSTACLE_TYPES.WIDE:
                this.width = 80 + Math.random() * 40;
                this.height = 30 + Math.random() * 20;
                break;
            case OBSTACLE_TYPES.FLOATING:
                this.width = 60 + Math.random() * 30;
                this.height = 20 + Math.random() * 20;
                break;
            case OBSTACLE_TYPES.SPIKY:
                this.width = 40 + Math.random() * 30;
                this.height = 40 + Math.random() * 30;
                this.addSpikes = true;
                break;
            default:
                this.width = width;
                this.height = 40 + Math.random() * 30;
        }
        
        const colors = [
            { main: '#4ecdc4', accent: '#2eada4' },
            { main: '#ff6b6b', accent: '#ff4b4b' },
            { main: '#95e1d3', accent: '#75c1b3' },
            { main: '#ffd93d', accent: '#ffb73d' },
            { main: '#6c5ce7', accent: '#4c3cc7' }
        ];
        const colorPair = colors[Math.floor(Math.random() * colors.length)];
        this.color = colorPair.main;
        this.accentColor = colorPair.accent;
    }

    draw() {
        ctx.save();
        
        // Calculate floating effect for floating platforms
        if (this.type === OBSTACLE_TYPES.FLOATING) {
            this.hoverOffset += this.hoverSpeed;
            this.y += Math.sin(this.hoverOffset) * 1.5;
        }
        
        // Draw shadow
        ctx.beginPath();
        if (this.type === OBSTACLE_TYPES.FLOATING) {
            const shadowY = canvas.height - 50;
            const scale = 0.5 - (shadowY - this.y) / 400;
            ctx.ellipse(
                this.x + this.width/2,
                shadowY,
                this.width/2 * scale,
                5 * scale,
                0,
                0,
                Math.PI * 2
            );
        } else {
            ctx.roundRect(this.x + 5, this.y + 5, this.width, this.height, this.roundness);
        }
        ctx.fillStyle = 'rgba(0, 0, 0, 0.2)';
        ctx.fill();

        // Draw main platform
        ctx.beginPath();
        
        if (this.type === OBSTACLE_TYPES.TALL) {
            // Draw tall obstacle with curved top
            const radius = this.width / 2;
            ctx.moveTo(this.x, this.y + this.height);
            ctx.lineTo(this.x, this.y + radius);
            ctx.quadraticCurveTo(this.x, this.y, this.x + radius, this.y);
            ctx.lineTo(this.x + this.width - radius, this.y);
            ctx.quadraticCurveTo(this.x + this.width, this.y, this.x + this.width, this.y + radius);
            ctx.lineTo(this.x + this.width, this.y + this.height);
            ctx.closePath();
        } else {
            ctx.roundRect(this.x, this.y, this.width, this.height, this.roundness);
        }
        
        const gradient = ctx.createLinearGradient(this.x, this.y, this.x + this.width, this.y + this.height);
        gradient.addColorStop(0, this.color);
        gradient.addColorStop(1, this.accentColor);
        
        ctx.fillStyle = gradient;
        ctx.fill();

        // Add design elements based on type
        switch (this.type) {
            case OBSTACLE_TYPES.SPIKY:
                this.drawSpikes();
                break;
            case OBSTACLE_TYPES.TALL:
                this.drawWindowsPattern();
                break;
            case OBSTACLE_TYPES.FLOATING:
                this.drawFloatingPattern();
                break;
            case OBSTACLE_TYPES.WIDE:
                this.drawStripes();
                break;
            default:
                this.drawDefaultPattern();
        }
        
        ctx.restore();
    }

    drawSpikes() {
        const spikeHeight = 15;
        const spikeWidth = 10;
        const numSpikes = Math.floor(this.width / 15);
        
        ctx.beginPath();
        for (let i = 0; i < numSpikes; i++) {
            const spikeX = this.x + (i + 0.5) * (this.width / numSpikes);
            ctx.moveTo(spikeX - spikeWidth/2, this.y);
            ctx.lineTo(spikeX, this.y - spikeHeight);
            ctx.lineTo(spikeX + spikeWidth/2, this.y);
        }
        ctx.fillStyle = this.accentColor;
        ctx.fill();
    }

    drawWindowsPattern() {
        const windowSize = 12;
        const gap = 8;
        const rows = Math.floor((this.height - 20) / (windowSize + gap));
        const cols = 2;

        for (let row = 0; row < rows; row++) {
            for (let col = 0; col < cols; col++) {
                const x = this.x + 10 + col * (windowSize + gap);
                const y = this.y + 15 + row * (windowSize + gap);
                
                ctx.beginPath();
                ctx.roundRect(x, y, windowSize, windowSize, 2);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
                ctx.fill();
            }
        }
    }

    drawFloatingPattern() {
        // Add glowing effect
        ctx.beginPath();
        ctx.roundRect(this.x, this.y, this.width, this.height, this.roundness);
        ctx.strokeStyle = 'rgba(255, 255, 255, 0.5)';
        ctx.lineWidth = 2;
        ctx.stroke();

        // Add hover particles
        for (let i = 0; i < 3; i++) {
            const particleX = this.x + Math.random() * this.width;
            const particleY = this.y + this.height + Math.random() * 10;
            const particleSize = Math.random() * 3 + 1;
            
            ctx.beginPath();
            ctx.arc(particleX, particleY, particleSize, 0, Math.PI * 2);
            ctx.fillStyle = 'rgba(255, 255, 255, 0.6)';
            ctx.fill();
        }
    }

    drawStripes() {
        const stripeWidth = 20;
        const numStripes = Math.floor(this.width / stripeWidth);
        
        for (let i = 0; i < numStripes; i++) {
            if (i % 2 === 0) continue;
            
            ctx.beginPath();
            ctx.fillStyle = 'rgba(255, 255, 255, 0.2)';
            ctx.fillRect(
                this.x + i * stripeWidth,
                this.y,
                stripeWidth,
                this.height
            );
        }
    }

    drawDefaultPattern() {
        // Add highlight
        ctx.beginPath();
        ctx.roundRect(this.x + 2, this.y + 2, this.width * 0.3, this.height * 0.1, this.roundness);
        ctx.fillStyle = 'rgba(255, 255, 255, 0.2)';
        ctx.fill();
    }

    update() {
        this.speed = gameSpeed;
        this.x -= this.speed;
        this.draw();
    }

    getBounds() {
        const bounds = {
            x: this.x,
            y: this.y,
            width: this.width,
            height: this.height
        };

        if (this.type === OBSTACLE_TYPES.SPIKY) {
            bounds.y -= 15; // Account for spikes in collision
            bounds.height += 15;
        }

        return bounds;
    }
}

// Add after Platform class
class Bird {
    x: number;
    y: number;
    width: number;
    height: number;
    color: string;
    wingOffset: number;
    wingSpeed: number;
    wingDirection: number;
    verticalOffset: number;
    verticalSpeed: number;

    constructor() {
        this.width = 30;
        this.height = 25;
        this.x = canvas.width;
        this.y = 100 + Math.random() * (canvas.height - 250);
        this.color = '#ff9f43';
        this.wingOffset = 0;
        this.wingSpeed = 0.2;
        this.wingDirection = 1;
        this.verticalOffset = 0;
        this.verticalSpeed = 0.02;
    }

    draw() {
        ctx.save();
        ctx.translate(this.x + this.width / 2, this.y + this.height / 2 + Math.sin(this.verticalOffset) * 15);

        // Draw shadow
        ctx.beginPath();
        ctx.ellipse(0, 50, this.width / 3, 5, 0, 0, Math.PI * 2);
        ctx.fillStyle = 'rgba(0, 0, 0, 0.2)';
        ctx.fill();

        // Draw body
        ctx.beginPath();
        ctx.ellipse(0, 0, this.width / 2, this.height / 2, 0, 0, Math.PI * 2);
        const gradient = ctx.createLinearGradient(-this.width/2, -this.height/2, this.width/2, this.height/2);
        gradient.addColorStop(0, this.color);
        gradient.addColorStop(1, '#f39c12');
        ctx.fillStyle = gradient;
        ctx.fill();

        // Draw wings with gradient
        const wingGradient = ctx.createLinearGradient(0, -15, 0, 0);
        wingGradient.addColorStop(0, '#f39c12');
        wingGradient.addColorStop(1, this.color);
        ctx.fillStyle = wingGradient;
        ctx.strokeStyle = this.color;
        ctx.lineWidth = 3;

        // Left wing
        ctx.beginPath();
        ctx.moveTo(-5, 0);
        ctx.quadraticCurveTo(
            -20, -15 + this.wingOffset,
            -10, 0
        );
        ctx.fill();
        ctx.stroke();

        // Right wing
        ctx.beginPath();
        ctx.moveTo(5, 0);
        ctx.quadraticCurveTo(
            20, -15 + this.wingOffset,
            10, 0
        );
        ctx.fill();
        ctx.stroke();

        // Draw eyes
        ctx.fillStyle = '#2d3436';
        ctx.beginPath();
        ctx.arc(-5, -2, 2, 0, Math.PI * 2);
        ctx.fill();

        // Draw beak
        ctx.beginPath();
        ctx.moveTo(this.width/4, 0);
        ctx.lineTo(this.width/4 + 8, 2);
        ctx.lineTo(this.width/4, 4);
        ctx.fillStyle = '#e17055';
        ctx.fill();

        ctx.restore();
    }

    update() {
        this.x -= gameSpeed * 0.8;
        this.verticalOffset = frame * this.verticalSpeed;
        this.wingOffset += this.wingSpeed * this.wingDirection;
        if (Math.abs(this.wingOffset) > 10) {
            this.wingDirection *= -1;
        }
        this.draw();
    }

    getBounds() {
        return {
            x: this.x + 5,
            y: this.y + Math.sin(this.verticalOffset) * 15 + 5,
            width: this.width - 10,
            height: this.height - 10
        };
    }
}

// Add enemy spawn control variables
let enemySpawnCooldown = 180; // 3 seconds initial cooldown
const MIN_ENEMY_SPAWN_INTERVAL = 180; // Minimum 3 seconds between spawns
const MAX_ENEMY_SPAWN_INTERVAL = 300; // Maximum 5 seconds between spawns
const MAX_SIMULTANEOUS_ENEMIES = 2; // Maximum enemies on screen at once

// Update generatePlatform function
function generatePlatform() {
    const minWidth = 40;
    const maxWidth = 100;
    const width = Math.random() * (maxWidth - minWidth) + minWidth;
    
    // Adjust gap based on game speed and obstacle type
    const minGap = 180 + gameSpeed * 10;
    const maxGap = 350 + gameSpeed * 10;
    const gap = Math.random() * (maxGap - minGap) + minGap;
    
    const lastObstacle = obstacles[obstacles.length - 1];
    const x = lastObstacle ? lastObstacle.getBounds().x + lastObstacle.getBounds().width + gap : canvas.width;
    const y = canvas.height - 50 - Math.random() * 30;
    
    // Increase bird frequency with score
    const birdChance = Math.min(0.3 + score / 100, 0.5);
    
    if (Math.random() < birdChance) {
        obstacles.push(new Bird());
    } else {
        const platform = new Platform(x, y, width);
        obstacles.push(platform);
    }

    // Add walking enemy with controlled spawning
    if (score > 30 && // Only start spawning after score 30
        walkingEnemies.length < MAX_SIMULTANEOUS_ENEMIES && // Limit max enemies
        enemySpawnCooldown <= 0 && // Check cooldown
        Math.random() < 0.4) { // 40% chance to spawn
        walkingEnemies.push(new WalkingEnemy());
        // Set random cooldown between min and max interval
        enemySpawnCooldown = Math.random() * (MAX_ENEMY_SPAWN_INTERVAL - MIN_ENEMY_SPAWN_INTERVAL) + MIN_ENEMY_SPAWN_INTERVAL;
    }
}

// Character selection function
export function setSelectedCharacter(character: keyof typeof characters) {
    selectedCharacter = character;
    const characterConfig = characters[character];
    
    // Reset game state completely
    score = 0;
    gameSpeed = INITIAL_GAME_SPEED;
    gameOver = false;
    frame = 0;
    
    // Clear all arrays
    obstacles = [];
    particles = [];
    darts = [];
    hammers = []; // Add this line
    
    // Reset player to initial state
    player.x = 100;
    player.y = canvas.height - 90;
    player.width = characterConfig.size.width;
    player.height = characterConfig.size.height;
    player.color = characterConfig.color;
    player.gravity = characterConfig.physics.gravity;
    player.jumpForce = characterConfig.physics.jumpForce;
    player.velocityY = 0;
    player.jumping = false;
    player.doubleJumped = false;
    player.rotation = 0;
    player.squash = 1;
    player.isParachuteActive = false; // Reset parachute state
    player.parachuteDeployed = false; // Reset parachute deployed state
    
    // Reset teleport/phase properties
    player.isTeleporting = false;
    player.teleportCooldown = 0;
    player.teleportDuration = 0;
    player.teleportParticles = [];
    player.phaseAlpha = 1;
    player.phaseDirection = 1;
    
    // Reset UI elements
    if (scoreEl) {
        scoreEl.textContent = '0';
    }
    
    if (gameOverEl) {
        gameOverEl.style.display = 'none';
        gameOverEl.style.opacity = '0';
        gameOverEl.classList.add('hidden');
    }

    // Clear and restart game loop
    if (gameLoop) {
        clearInterval(gameLoop);
    }
    gameLoop = window.setInterval(update, 1000/60);
    gameStarted = true;

    // Generate initial platform
    generatePlatform();

    // Reset enemies
    walkingEnemies = [];
    enemyProjectiles = [];
    slimeProjectiles = [];
}

// Add particle type definition before the particles array
type Particle = {
    x: number;
    y: number;
    size: number;
    speedX: number;
    speedY: number;
    life: number;
    color: string;
    rotation?: number;
    rotationSpeed?: number;
    gravity?: number;
    flutter?: number;
    fadeSpeed?: number;
    glow?: boolean;
};

// Update particles array declaration
let particles: Particle[] = [];

// Update updatePlayer function
function updatePlayer() {
    // Apply gravity with parachute effect
    if (selectedCharacter === 'classic' && player.isParachuteActive) {
        const classicConfig = characters.classic;
        player.velocityY += classicConfig.physics.gravity * 0.3; // Reduced gravity with parachute
        player.velocityY = Math.min(player.velocityY, 2); // Cap falling speed with parachute
        
        // Add floating particles
        if (frame % 3 === 0) {
            particles.push({
                x: player.x + player.width/2 + (Math.random() - 0.5) * player.width,
                y: player.y - player.height/2,
                size: Math.random() * 2 + 1,
                speedX: (Math.random() - 0.5),
                speedY: Math.random() * 0.5,
                life: 1,
                color: characters.classic.color,
                fadeSpeed: 0.02
            });
        }
    } else {
        player.velocityY += characters[selectedCharacter as keyof typeof characters].physics.gravity;
    }

    // Update player position
    player.y += player.velocityY;

    // Ground collision
    if (player.y > canvas.height - 90) {
        player.y = canvas.height - 90;
        player.velocityY = 0;
        player.jumping = false;
        player.doubleJumped = false;
        player.isParachuteActive = false;
        player.parachuteDeployed = false; // Reset parachute state
        player.rotation = 0;
        player.squash = 1.2;
    }

    // Update squash effect
    player.squash += (1 - player.squash) * 0.2;

    // Update rotation for ninja flip
    if (player.doubleJumped) {
        player.rotation += 0.2;
        if (player.rotation >= Math.PI * 2) {
            player.rotation = 0;
        }
    }

    // Update teleport cooldown
    if (player.teleportCooldown > 0) {
        player.teleportCooldown--;
    }

    // Update teleport duration and phase animation
    if (player.teleportDuration > 0) {
        player.teleportDuration--;
        
        // Phase animation
        player.phaseAlpha += 0.05 * player.phaseDirection;
        
        // Control phase animation transitions
        if (player.teleportDuration <= 30) { // Start phasing back in during last 0.5 seconds
            player.phaseDirection = 1;
        }
        
        // Clamp alpha values
        player.phaseAlpha = Math.max(0.2, Math.min(1, player.phaseAlpha));
        
        if (player.teleportDuration <= 0) {
            player.isTeleporting = false;
            player.phaseAlpha = 1;
            
            // Create phase end particles
            for (let i = 0; i < 20; i++) {
                const angle = (i / 20) * Math.PI * 2;
                const speed = 2 + Math.random() * 4;
                player.teleportParticles.push({
                    x: player.x + player.width / 2,
                    y: player.y + player.height / 2,
                    size: Math.random() * 4 + 2,
                    speedX: Math.cos(angle) * speed,
                    speedY: Math.sin(angle) * speed,
                    life: 1,
                    color: '#ff1493', // Deep pink for end particles
                    fadeSpeed: 0.03,
                    glow: true
                });
            }
        }
    }

    // Update teleport particles with enhanced effects
    player.teleportParticles = player.teleportParticles.filter(particle => {
        particle.x += particle.speedX;
        particle.y += particle.speedY;
        particle.speedX *= 0.95; // Add slight deceleration
        particle.speedY *= 0.95;
        particle.life -= particle.fadeSpeed || 0.02;

        if (particle.life > 0) {
            ctx.save();
            if (particle.glow) {
                ctx.shadowColor = particle.color;
                ctx.shadowBlur = 15;
            }
            ctx.globalAlpha = particle.life * (player.isTeleporting ? 0.7 : 1);
            ctx.fillStyle = particle.color;
            ctx.beginPath();
            ctx.arc(particle.x, particle.y, particle.size * (1 + Math.sin(frame * 0.2) * 0.2), 0, Math.PI * 2);
            ctx.fill();
            ctx.restore();
            return true;
        }
        return false;
    });

    // Create continuous phase particles
    if (player.isTeleporting && frame % 2 === 0) {
        const angle = Math.random() * Math.PI * 2;
        const distance = Math.random() * player.width * 0.8;
        player.teleportParticles.push({
            x: player.x + player.width / 2 + Math.cos(angle) * distance,
            y: player.y + player.height / 2 + Math.sin(angle) * distance,
            size: Math.random() * 3 + 1,
            speedX: (Math.random() - 0.5) * 1,
            speedY: (Math.random() - 0.5) * 1,
            life: 0.5,
            color: Math.random() > 0.5 ? '#ff69b4' : '#ff1493', // Alternate between hot pink and deep pink
            fadeSpeed: 0.05,
            glow: true
        });
    }
}

// Update drawPlayer function
function drawPlayer() {
    const characterConfig = characters[selectedCharacter as keyof typeof characters];
    
    ctx.save();
    
    // Apply phase effects for alien
    if (selectedCharacter === 'alien' && player.isTeleporting) {
        ctx.globalAlpha = player.phaseAlpha;
        // Change color during phase to cool pink
        characterConfig.color = '#ff69b4'; // Hot pink
        characterConfig.accentColor = '#ff1493'; // Deep pink
        characterConfig.effects.trailColor = 'rgba(255, 105, 180, 0.3)';
    } else if (selectedCharacter === 'alien') {
        // Reset to original colors
        characterConfig.color = '#00b894';
        characterConfig.accentColor = '#00cec9';
        characterConfig.effects.trailColor = 'rgba(0, 184, 148, 0.3)';
    }
    
    ctx.translate(
        player.x + player.width / 2,
        player.y + player.height / 2
    );
    ctx.rotate(player.rotation);
    ctx.scale(1 / player.squash, player.squash);

    // Enhanced shadow effect
    ctx.shadowColor = 'rgba(0, 0, 0, 0.3)';
    ctx.shadowBlur = 15;
    ctx.shadowOffsetY = 8;

    // Add floating animation
    const floatOffset = Math.sin(frame * 0.05) * 3;
    ctx.translate(0, floatOffset);

    // Draw character based on shape with enhanced effects
    switch(characterConfig.shape) {
        case 'round': {
            // Classic character with enhanced gradient and pulse effect
            const pulseSize = 1 + Math.sin(frame * 0.1) * 0.05;
            const gradientClassic = ctx.createRadialGradient(0, 0, 0, 0, 0, (player.width/2) * pulseSize);
            gradientClassic.addColorStop(0, characterConfig.color);
            gradientClassic.addColorStop(0.7, characterConfig.accentColor);
            gradientClassic.addColorStop(1, characterConfig.color);
            ctx.fillStyle = gradientClassic;
            
            // Draw parachute if active
            if (player.isParachuteActive) {
                const parachuteWidth = player.width * 2.2;
                const parachuteHeight = player.height * 1.8;
                const windEffect = Math.sin(frame * 0.05) * 5;
                
                // Draw parachute canopy with improved design
                ctx.beginPath();
                ctx.moveTo(-parachuteWidth/2 + windEffect, -player.height/2 - parachuteHeight * 0.8);
                
                // Create smooth dome shape
                ctx.bezierCurveTo(
                    -parachuteWidth/2 + windEffect, -player.height/2 - parachuteHeight,
                    -parachuteWidth/3 + windEffect, -player.height/2 - parachuteHeight * 1.1,
                    0 + windEffect, -player.height/2 - parachuteHeight
                );
                ctx.bezierCurveTo(
                    parachuteWidth/3 + windEffect, -player.height/2 - parachuteHeight * 1.1,
                    parachuteWidth/2 + windEffect, -player.height/2 - parachuteHeight,
                    parachuteWidth/2 + windEffect, -player.height/2 - parachuteHeight * 0.8
                );
                
                // Create multi-layered gradient for parachute
                const parachuteGradient = ctx.createRadialGradient(
                    0, -player.height/2 - parachuteHeight * 0.9,
                    0,
                    0, -player.height/2 - parachuteHeight * 0.9,
                    parachuteWidth/2
                );
                parachuteGradient.addColorStop(0, characterConfig.color);
                parachuteGradient.addColorStop(0.6, characterConfig.accentColor);
                parachuteGradient.addColorStop(1, characterConfig.color);
                ctx.fillStyle = parachuteGradient;
                
                // Add shadow for depth
                ctx.shadowColor = 'rgba(0, 0, 0, 0.2)';
                ctx.shadowBlur = 10;
                ctx.shadowOffsetY = 5;
                
                ctx.fill();
                
                // Draw air vents
                const ventCount = 5;
                for (let i = 0; i < ventCount; i++) {
                    const ventX = -parachuteWidth/2 + (parachuteWidth/(ventCount-1)) * i + windEffect;
                    const ventY = -player.height/2 - parachuteHeight * 0.85;
                    const ventSize = player.width * 0.15;
                    
                    ctx.beginPath();
                    ctx.arc(ventX, ventY, ventSize, 0, Math.PI * 2);
                    ctx.fillStyle = 'rgba(255, 255, 255, 0.2)';
                    ctx.fill();
                }
                
                // Draw single central string with wave effect and hands
                ctx.beginPath();
                const stringWave = Math.sin(frame * 0.1) * 3;
                
                // Left string with hand
                ctx.beginPath();
                ctx.moveTo(-player.width/4, -player.height/2);
                ctx.quadraticCurveTo(
                    stringWave - player.width/3, -player.height/2 - parachuteHeight * 0.4,
                    -parachuteWidth/3 + windEffect, -player.height/2 - parachuteHeight * 0.8
                );
                ctx.strokeStyle = characterConfig.accentColor;
                ctx.lineWidth = 2;
                ctx.stroke();

                // Right string with hand
                ctx.beginPath();
                ctx.moveTo(player.width/4, -player.height/2);
                ctx.quadraticCurveTo(
                    stringWave + player.width/3, -player.height/2 - parachuteHeight * 0.4,
                    parachuteWidth/3 + windEffect, -player.height/2 - parachuteHeight * 0.8
                );
                ctx.stroke();

                // Draw left hand
                ctx.beginPath();
                ctx.fillStyle = characterConfig.color;
                ctx.arc(-player.width/4, -player.height/2, 6, 0, Math.PI * 2);
                ctx.fill();
                
                // Add fingers to left hand
                for(let i = 0; i < 3; i++) {
                    ctx.beginPath();
                    ctx.arc(
                        -player.width/4 - 3 + (i * 2), 
                        -player.height/2 - 4,
                        2,
                        0,
                        Math.PI * 2
                    );
                    ctx.fill();
                }

                // Draw right hand
                ctx.beginPath();
                ctx.arc(player.width/4, -player.height/2, 6, 0, Math.PI * 2);
                ctx.fill();
                
                // Add fingers to right hand
                for(let i = 0; i < 3; i++) {
                    ctx.beginPath();
                    ctx.arc(
                        player.width/4 + 3 - (i * 2), 
                        -player.height/2 - 4,
                        2,
                        0,
                        Math.PI * 2
                    );
                    ctx.fill();
                }
            }
            
            // Draw main body with glow
            ctx.shadowColor = characterConfig.color;
            ctx.shadowBlur = 20;
            ctx.beginPath();
            ctx.arc(0, 0, player.width/2, 0, Math.PI * 2);
            ctx.fill();

            // Add dynamic shine effect
            const shineAngle = frame * 0.02;
            ctx.beginPath();
            ctx.arc(
                Math.cos(shineAngle) * player.width/4,
                Math.sin(shineAngle) * player.width/4,
                player.width/4,
                0,
                Math.PI * 2
            );
            ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
            ctx.fill();
            break;
        }
        case 'diamond': {
            // Ninja character with enhanced metallic effect and trail
            const pulseSize = 1 + Math.sin(frame * 0.08) * 0.05;
            const gradientNinja = ctx.createLinearGradient(
                -player.width/2 * pulseSize, -player.height/2 * pulseSize,
                player.width/2 * pulseSize, player.height/2 * pulseSize
            );
            gradientNinja.addColorStop(0, characterConfig.color);
            gradientNinja.addColorStop(0.5, characterConfig.accentColor);
            gradientNinja.addColorStop(1, characterConfig.color);
            ctx.fillStyle = gradientNinja;
            
            // Draw main body with shadow trail
            if (player.velocityY !== 0) {
                ctx.shadowColor = characterConfig.effects.trailColor;
                ctx.shadowBlur = 30;
            }
            
            ctx.beginPath();
            ctx.moveTo(0, -player.height/2 * pulseSize);
            ctx.lineTo(player.width/2 * pulseSize, 0);
            ctx.lineTo(0, player.height/2 * pulseSize);
            ctx.lineTo(-player.width/2 * pulseSize, 0);
            ctx.closePath();
            ctx.fill();

            // Add dynamic ninja band with glowing edge
            const bandGlow = Math.abs(Math.sin(frame * 0.1)) * 10;
            ctx.shadowColor = characterConfig.bandColor;
            ctx.shadowBlur = bandGlow;
            ctx.fillStyle = characterConfig.bandColor;
            ctx.fillRect(-player.width/2 - 5, -player.height/6, player.width + 10, player.height/6);

            // Add short head tie at the tip
            const tieLength = player.height * 0.25; // Shorter length
            const tieWidth = player.width * 0.15; // Slightly thinner
            const windEffect = Math.sin(frame * 0.1) * 5; // Reduced wind effect
            
            // Draw the short tie from the top point
            ctx.beginPath();
            ctx.moveTo(0, -player.height/2 * pulseSize); // Start from the tip of the diamond
            
            // Create two simple curves for the tie ends
            ctx.lineTo(-tieWidth/2 + windEffect, -player.height/2 * pulseSize + tieLength);
            ctx.lineTo(0, -player.height/2 * pulseSize + tieLength * 0.8);
            ctx.lineTo(tieWidth/2 + windEffect, -player.height/2 * pulseSize + tieLength);
            ctx.closePath();
            
            ctx.fillStyle = characterConfig.bandColor;
            ctx.fill();

            // Add metallic shine effect
            const shinePos = (Math.sin(frame * 0.05) + 1) / 2;
            const gradient = ctx.createLinearGradient(
                -player.width/2, -player.height/2,
                player.width/2, player.height/2
            );
            gradient.addColorStop(Math.max(0, shinePos - 0.2), 'rgba(255, 255, 255, 0)');
            gradient.addColorStop(shinePos, 'rgba(255, 255, 255, 0.3)');
            gradient.addColorStop(Math.min(1, shinePos + 0.2), 'rgba(255, 255, 255, 0)');
            ctx.fillStyle = gradient;
            ctx.fill();
            break;
        }
        case 'square': {
            // Steampunk Robot character
            const gearRotation = frame * 0.02;
            
            ctx.save();
            
            // Steam effect
            if (frame % 30 === 0) {
                for (let i = 0; i < 3; i++) {
                    particles.push({
                        x: player.x + player.width * 0.3,
                        y: player.y + player.height * 0.2,
                        size: Math.random() * 4 + 2,
                        speedX: (Math.random() - 0.5) * 1,
                        speedY: -Math.random() * 2,
                        life: 1,
                        color: 'rgba(255, 255, 255, 0.8)',
                        fadeSpeed: 0.02
                    });
                }
            }

            // Main body with brass and copper tones
            const bodyGradient = ctx.createLinearGradient(
                -player.width/2, -player.height/2,
                player.width/2, player.height/2
            );
            bodyGradient.addColorStop(0, characterConfig.effects.metalSheen.primary);
            bodyGradient.addColorStop(0.5, characterConfig.color);
            bodyGradient.addColorStop(1, characterConfig.effects.metalSheen.secondary);
            
            // Draw segmented body
            ctx.fillStyle = bodyGradient;
            
            // Upper body (chest)
            ctx.beginPath();
            ctx.roundRect(
                -player.width/2, -player.height/2,
                player.width, player.height * 0.5,
                10
            );
            ctx.fill();
            
            // Lower body (waist)
            ctx.beginPath();
            ctx.roundRect(
                -player.width/2 * 0.8, -player.height/2 + player.height * 0.4,
                player.width * 0.8, player.height * 0.3,
                8
            );
            ctx.fill();

            // Motorized Feet System
            const legGap = player.width * 0.3;

       

            // Keep only the roller
            drawRoller();

            // Add steam vents near feet
            if (frame % 20 === 0) {
                [-legGap/2, legGap/2].forEach(x => {
                    for (let i = 0; i < 2; i++) {
                        particles.push({
                            x: player.x + player.width/2 + x,
                            y: player.y + player.height,
                            size: Math.random() * 3 + 2,
                            speedX: (Math.random() - 0.5) * 1,
                            speedY: -Math.random() * 2,
                            life: 1,
                            color: 'rgba(255, 255, 255, 0.8)',
                            fadeSpeed: 0.03
                        });
                    }
                });
            }

            // Rotating gears
            function drawGear(x: number, y: number, size: number, teeth: number, rotation: number) {
                ctx.save();
                ctx.translate(x, y);
                ctx.rotate(rotation);
                
                // Outer gear
            ctx.beginPath();
                for (let i = 0; i < teeth; i++) {
                    const angle = (i / teeth) * Math.PI * 2;
                    const outerX = Math.cos(angle) * (size + 4);
                    const outerY = Math.sin(angle) * (size + 4);
                    const innerX = Math.cos(angle) * size;
                    const innerY = Math.sin(angle) * size;
                    
                    if (i === 0) {
                        ctx.moveTo(outerX, outerY);
                    } else {
                        ctx.lineTo(outerX, outerY);
                    }
                    ctx.lineTo(innerX, innerY);
                }
                ctx.closePath();
                
                const gearGradient = ctx.createRadialGradient(0, 0, size * 0.5, 0, 0, size + 4);
                ctx.fillStyle = gearGradient;
            ctx.fill();
            
                // Center hole
            ctx.beginPath();
                ctx.arc(0, 0, size * 0.2, 0, Math.PI * 2);
                ctx.fillStyle = '#2c3e50';
            ctx.fill();

                ctx.restore();
            }

            // Draw multiple gears
            drawGear(-player.width * 0.3, -player.height * 0.2, 12, 8, gearRotation);
            drawGear(player.width * 0.3, -player.height * 0.2, 10, 6, -gearRotation * 1.5);
            drawGear(0, player.height * 0.2, 8, 7, gearRotation * 0.8);

            // Pressure gauge
            const gaugeX = 0;
            const gaugeY = -player.height * 0.1;
            const gaugeSize = player.width * 0.15;
            
            ctx.beginPath();
            ctx.arc(gaugeX, gaugeY, gaugeSize, 0, Math.PI * 2);
            const gaugeGradient = ctx.createRadialGradient(
                gaugeX, gaugeY, gaugeSize * 0.5,
                gaugeX, gaugeY, gaugeSize
            );
            gaugeGradient.addColorStop(0, '#2c3e50');
            gaugeGradient.addColorStop(1, '#34495e');
            ctx.fillStyle = gaugeGradient;
                ctx.fill();

            // Gauge needle
            const needleAngle = (Math.sin(frame * 0.05) + 1) * Math.PI * 0.75;
                ctx.beginPath();
            ctx.moveTo(gaugeX, gaugeY);
            ctx.lineTo(
                gaugeX + Math.cos(needleAngle) * gaugeSize * 0.8,
                gaugeY + Math.sin(needleAngle) * gaugeSize * 0.8
            );
            ctx.strokeStyle = '#e74c3c';
                ctx.lineWidth = 2;
                ctx.stroke();

            // Enhanced mechanical eyes
            function drawMechanicalEye(x: number, y: number, size: number) {
                // Outer ring with soft glow
                ctx.beginPath();
                ctx.arc(x, y, size * 1.3, 0, Math.PI * 2);
                const ringGradient = ctx.createRadialGradient(x, y, size * 0.8, x, y, size * 1.3);
                ringGradient.addColorStop(0, characterConfig.eyeColor);
                ringGradient.addColorStop(1, 'rgba(241, 196, 15, 0)');
                ctx.fillStyle = ringGradient;
                ctx.fill();

                // Main eye with friendly shape
                ctx.beginPath();
                ctx.arc(x, y, size, 0, Math.PI * 2);
                const eyeGradient = ctx.createRadialGradient(x, y, 0, x, y, size);
                eyeGradient.addColorStop(0, '#ffffff');
                eyeGradient.addColorStop(0.6, characterConfig.eyeColor);
                eyeGradient.addColorStop(1, '#f39c12');
                ctx.fillStyle = eyeGradient;
                ctx.fill();

                // Add cute expression
                const happiness = Math.sin(frame * 0.02) * 0.5 + 0.5;
                const eyeY = y - size * 0.2 * happiness;
                
                // Draw curved eyelid for friendly look
                    ctx.beginPath();
                ctx.moveTo(x - size, eyeY);
                ctx.quadraticCurveTo(x, eyeY - size * 0.5 * happiness, x + size, eyeY);
                ctx.strokeStyle = characterConfig.eyeColor;
                ctx.lineWidth = 2;
                ctx.stroke();

                // Friendly shine effect
                const shineSize = size * 0.4;
                const shineOffset = Math.sin(frame * 0.05) * size * 0.1;
                ctx.beginPath();
                ctx.arc(x - size * 0.2 + shineOffset, y - size * 0.2, shineSize, 0, Math.PI * 2);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.8)';
                    ctx.fill();
                
                // Add small twinkle
                ctx.beginPath();
                ctx.arc(x + size * 0.3, y - size * 0.3, size * 0.15, 0, Math.PI * 2);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.6)';
                ctx.fill();
            }

            // Draw rounder, friendlier body
            ctx.beginPath();
            ctx.roundRect(
                -player.width/2, -player.height/2,
                player.width, player.height * 0.6,
                20
            );
            ctx.fill();

            // Draw large roller
            function drawRoller() {
                const rollerWidth = player.width * 1.4;
                const rollerHeight = player.height * 0.35;
                const rollerY = player.height * 0.4;
                
                ctx.save();
                ctx.translate(0, rollerY);

                // Main roller body with enhanced gradient
                const rollerGradient = ctx.createLinearGradient(
                    -rollerWidth/2, -rollerHeight/2,
                    rollerWidth/2, rollerHeight/2
                );
                rollerGradient.addColorStop(0, '#2e86de');
                rollerGradient.addColorStop(0.3, '#ff9f43');
                rollerGradient.addColorStop(0.7, '#ff9f43');
                rollerGradient.addColorStop(1, '#2e86de');
                
                // Draw main cylinder
                ctx.fillStyle = rollerGradient;
                ctx.beginPath();
                ctx.roundRect(
                    -rollerWidth/2, -rollerHeight/2,
                    rollerWidth, rollerHeight,
                    rollerHeight/2
                );
                ctx.fill();

                // Add mechanical grooves with improved pattern
                const grooves = 16;
                for (let i = 0; i < grooves; i++) {
                    const x = -rollerWidth/2 + (rollerWidth * (i + 0.5) / grooves);
                    
                    ctx.beginPath();
                    ctx.moveTo(x, -rollerHeight/2 + 4);
                    ctx.lineTo(x, rollerHeight/2 - 4);
                    ctx.strokeStyle = 'rgba(255, 255, 255, 0.2)';
                    ctx.lineWidth = 2;
                    ctx.stroke();
                }

                // Add rotating mechanical pattern
                const rotation = frame * 0.1;
                const segments = 8;
                for (let i = 0; i < segments; i++) {
                    const angle = (i / segments) * Math.PI * 2 + rotation;
                    const radius = rollerHeight * 0.7;
                    const x1 = -rollerWidth/3 + Math.cos(angle) * radius;
                    const y1 = Math.sin(angle) * radius;
                    const x2 = rollerWidth/3 + Math.cos(angle) * radius;
                    const y2 = Math.sin(angle) * radius;
                    
                    ctx.beginPath();
                    ctx.moveTo(x1, y1);
                    ctx.lineTo(x2, y2);
                    ctx.strokeStyle = 'rgba(255, 255, 255, 0.15)';
                    ctx.lineWidth = 3;
                    ctx.stroke();
                }

                // Add subtle metallic shine
                const shineGradient = ctx.createLinearGradient(
                    -rollerWidth/2, -rollerHeight/2,
                    rollerWidth/2, rollerHeight/2
                );
                shineGradient.addColorStop(0, 'rgba(255, 255, 255, 0.2)');
                shineGradient.addColorStop(0.5, 'rgba(255, 255, 255, 0)');
                shineGradient.addColorStop(1, 'rgba(255, 255, 255, 0.2)');
                ctx.fillStyle = shineGradient;
                ctx.fill();

                ctx.restore();
            }

            drawRoller();

            // Draw end effectors
            function drawEndEffector() {
                const baseWidth = player.width * 0.15;  // Slightly narrower base
                const baseHeight = player.height * 0.25; // Taller base
                const gripperX = player.width * 0.5;  // Extended forward
                const gripperY = -player.height * 0.1;
                
                ctx.save();
                ctx.translate(gripperX, gripperY);
                ctx.rotate(-Math.PI/2); // Rotate to point forward

                // Base mount with new colors
                const mountGradient = ctx.createLinearGradient(
                    -baseWidth/2, 0,
                    baseWidth/2, baseHeight
                );
                mountGradient.addColorStop(0, '#ff9f43'); // Orange
                mountGradient.addColorStop(1, '#2e86de'); // Blue
                
                ctx.fillStyle = mountGradient;
                ctx.beginPath();
                ctx.roundRect(
                    -baseWidth/2, 0,
                    baseWidth, baseHeight,
                    4
                );
                ctx.fill();

                // Enhanced single gripper
                const fingerLength = baseHeight * 0.8;
                const fingerWidth = baseWidth * 0.3;
                const gripperOpen = Math.sin(frame * 0.1) * 0.5 + 0.5;
                const fingerSpacing = baseWidth * 0.8 * gripperOpen;

                // Draw two parts of the gripper
                [-1, 1].forEach(fingerSide => {
                    const fingerGradient = ctx.createLinearGradient(
                        0, 0,
                        fingerWidth, fingerLength
                    );
                    fingerGradient.addColorStop(0, '#2e86de'); // Blue
                    fingerGradient.addColorStop(1, '#ff9f43'); // Orange
                    
                    ctx.fillStyle = fingerGradient;
                    ctx.beginPath();
                    ctx.roundRect(
                        fingerSide * fingerSpacing - fingerWidth/2,
                        baseHeight,
                        fingerWidth,
                        fingerLength,
                        3
                    );
                    ctx.fill();

                    // Add metallic highlight
                    ctx.beginPath();
                    ctx.roundRect(
                        fingerSide * fingerSpacing - fingerWidth/4,
                        baseHeight + fingerLength * 0.2,
                        fingerWidth/2,
                        fingerLength * 0.3,
                        2
                    );
                    ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
                    ctx.fill();
                });

                ctx.restore();
            }

            // Draw both end effectors
            drawEndEffector(); // Left gripper
            drawEndEffector();  // Right gripper

            // Add a cute smile
            const smileY = -player.height * 0.1;
            const smileWidth = player.width * 0.3;
            const happiness = Math.sin(frame * 0.02) * 0.5 + 0.5;
            
            ctx.beginPath();
            ctx.moveTo(-smileWidth, smileY);
            ctx.quadraticCurveTo(0, smileY + 15 * happiness, smileWidth, smileY);
            ctx.strokeStyle = characterConfig.accentColor;
            ctx.lineWidth = 3;
            ctx.lineCap = 'round';
            ctx.stroke();

            // Add rosy cheeks
            const cheekColor = 'rgba(46, 204, 113, 0.3)';
            const cheekSize = player.width * 0.12;
            const cheekY = -player.height * 0.05;
            
            ctx.beginPath();
            ctx.arc(-player.width * 0.25, cheekY, cheekSize, 0, Math.PI * 2);
            ctx.arc(player.width * 0.25, cheekY, cheekSize, 0, Math.PI * 2);
            ctx.fillStyle = cheekColor;
            ctx.fill();

            // Draw mechanical eyes
            const eyeSize = player.width * 0.15;
            const eyeSpacing = player.width * 0.25;
            const eyeY = -player.height * 0.25;
            
            drawMechanicalEye(-eyeSpacing, eyeY, eyeSize);
            drawMechanicalEye(eyeSpacing, eyeY, eyeSize);

            // Add hovering drone companion with pink theme
            function drawDrone() {
                const droneWidth = player.width * 0.4;
                const droneHeight = player.height * 0.2;
                const droneX = 0;
                const droneY = -player.height * 1.2;
                const hoverOffset = Math.sin(frame * 0.1) * 3;

                ctx.save();
                ctx.translate(droneX, droneY + hoverOffset);

                // Draw enhanced protective beam with pink glow
                const beamGradient = ctx.createLinearGradient(0, 0, 0, player.height * 0.7);
                beamGradient.addColorStop(0, 'rgba(255, 105, 180, 0.3)'); // Hot pink
                beamGradient.addColorStop(1, 'rgba(255, 20, 147, 0)'); // Deep pink fade
                ctx.fillStyle = beamGradient;
                ctx.beginPath();
                ctx.moveTo(-droneWidth/4, 0);
                ctx.lineTo(droneWidth/4, 0);
                ctx.lineTo(droneWidth/2, player.height * 0.7);
                ctx.lineTo(-droneWidth/2, player.height * 0.7);
                ctx.closePath();
                ctx.fill();

                // Add beam sparkles
                for (let i = 0; i < 3; i++) {
                    const sparkleY = Math.random() * player.height * 0.6;
                    const sparkleX = (Math.random() - 0.5) * droneWidth * 0.8;
                    const sparkleSize = Math.random() * 3 + 1;
                    ctx.beginPath();
                    ctx.arc(sparkleX, sparkleY, sparkleSize, 0, Math.PI * 2);
                    ctx.fillStyle = 'rgba(255, 182, 193, 0.8)'; // Light pink
                    ctx.fill();
                }

                // Drone body with pink gradient
                const bodyGradient = ctx.createLinearGradient(
                    -droneWidth/2, -droneHeight/2,
                    droneWidth/2, droneHeight/2
                );
                bodyGradient.addColorStop(0, '#ff69b4'); // Hot pink
                bodyGradient.addColorStop(1, '#ff1493'); // Deep pink
                
                ctx.fillStyle = bodyGradient;
            ctx.beginPath();
            ctx.roundRect(
                    -droneWidth/2, -droneHeight/2,
                    droneWidth, droneHeight,
                    8
                );
            ctx.fill();

                // Add metallic shine
                const shineGradient = ctx.createLinearGradient(
                    -droneWidth/2, -droneHeight/2,
                    droneWidth/2, droneHeight/2
                );
                shineGradient.addColorStop(0, 'rgba(255, 255, 255, 0.2)');
                shineGradient.addColorStop(0.5, 'rgba(255, 255, 255, 0)');
                shineGradient.addColorStop(1, 'rgba(255, 255, 255, 0.2)');
                ctx.fillStyle = shineGradient;
                ctx.fill();

                // Propeller housings with enhanced design
                const propellerPositions = [
                    {x: -droneWidth * 0.4, y: 0},
                    {x: droneWidth * 0.4, y: 0}
                ];

                propellerPositions.forEach(pos => {
                    // Housing gradient
                    const housingGradient = ctx.createRadialGradient(
                        pos.x, pos.y, 0,
                        pos.x, pos.y, droneHeight * 0.3
                    );
                    housingGradient.addColorStop(0, '#ff69b4');
                    housingGradient.addColorStop(1, '#c71585');
                    
                    ctx.fillStyle = housingGradient;
                    ctx.beginPath();
                    ctx.arc(pos.x, pos.y, droneHeight * 0.3, 0, Math.PI * 2);
                    ctx.fill();
                });

                // Spinning propellers with pink glow
                const propellerRotation = frame * 0.5;
                propellerPositions.forEach(pos => {
                    ctx.save();
                    ctx.translate(pos.x, pos.y);
                    ctx.rotate(propellerRotation);

                    // Add propeller glow
                    ctx.shadowColor = '#ff69b4';
                    ctx.shadowBlur = 10;

                for (let i = 0; i < 3; i++) {
                        ctx.rotate(Math.PI * 2 / 3);
                        const propGradient = ctx.createLinearGradient(
                            0, -droneHeight * 0.3,
                            0, droneHeight * 0.3
                        );
                        propGradient.addColorStop(0, '#ffc0cb'); // Pink
                        propGradient.addColorStop(1, '#ff69b4'); // Hot pink
                        ctx.fillStyle = propGradient;
            ctx.beginPath();
                        ctx.roundRect(-droneHeight * 0.1, -droneHeight * 0.3,
                            droneHeight * 0.2, droneHeight * 0.6, 4);
            ctx.fill();
                }
                    ctx.restore();
                });

                // Enhanced scanner/eye with pink theme
                const scannerSize = droneHeight * 0.3;
                const scannerGlow = Math.abs(Math.sin(frame * 0.1));
                
                // Scanner base
                ctx.beginPath();
                ctx.arc(0, 0, scannerSize, 0, Math.PI * 2);
                const scannerGradient = ctx.createRadialGradient(0, 0, 0, 0, 0, scannerSize);
                scannerGradient.addColorStop(0, '#ff69b4'); // Hot pink
                scannerGradient.addColorStop(0.6, '#ff1493'); // Deep pink
                scannerGradient.addColorStop(1, '#c71585'); // Medium violet red
                ctx.fillStyle = scannerGradient;
                ctx.fill();

                // Scanner glow effect
                ctx.shadowColor = '#ff69b4';
                ctx.shadowBlur = 15 + scannerGlow * 15;
                ctx.beginPath();
                ctx.arc(0, 0, scannerSize * 0.7, 0, Math.PI * 2);
                ctx.fillStyle = '#ff69b4';
                ctx.fill();

                // Add scanning line animation
                ctx.beginPath();
                ctx.moveTo(-scannerSize, 0);
                ctx.lineTo(scannerSize, 0);
                ctx.strokeStyle = `rgba(255, 255, 255, ${0.5 + scannerGlow * 0.5})`;
                ctx.lineWidth = 2;
                ctx.stroke();

                // Add status lights with pink theme
                [-1, 1].forEach(side => {
                    const lightGlow = Math.abs(Math.sin(frame * 0.1 + side));
                    ctx.beginPath();
                    ctx.arc(side * droneWidth * 0.3, -droneHeight * 0.2,
                        droneHeight * 0.1, 0, Math.PI * 2);
                    ctx.fillStyle = `rgba(255, 105, 180, ${0.5 + lightGlow * 0.5})`;
                    ctx.fill();

                    // Add light glow
                    ctx.shadowColor = '#ff69b4';
                    ctx.shadowBlur = 10 + lightGlow * 5;
                    ctx.beginPath();
                    ctx.arc(side * droneWidth * 0.3, -droneHeight * 0.2,
                        droneHeight * 0.05, 0, Math.PI * 2);
                    ctx.fillStyle = '#ffffff';
                    ctx.fill();
                });

                ctx.restore();
            }

            drawDrone();
            
            ctx.restore();
            break;
        }
        case 'oval': {
            // Alien character with enhanced ethereal effects
            const pulseSize = 1 + Math.sin(frame * 0.06) * 0.08;
            
            // Create multi-layered outer glow
            if (characterConfig.effects.hasGlow) {
                const glowIntensity = 0.5 + Math.sin(frame * 0.1) * 0.2;
                
                // Outer aura layers
                for (let i = 0; i < 4; i++) {
                    const layerSize = 1 + i * 0.15;
                    const opacity = (0.15 - i * 0.03) * glowIntensity;
                    
                    ctx.beginPath();
                    ctx.ellipse(0, 0,
                        player.width/2 * layerSize * pulseSize,
                        player.height/2 * layerSize * pulseSize,
                        frame * 0.01 + i * Math.PI/4, 0, Math.PI * 2
                    );
                    ctx.fillStyle = `rgba(${hexToRgb(characterConfig.color)?.r}, ${hexToRgb(characterConfig.color)?.g}, ${hexToRgb(characterConfig.color)?.b}, ${opacity})`;
                    ctx.fill();
                }
                
                // Add energy particles
                const particleCount = 8;
                for (let i = 0; i < particleCount; i++) {
                    const angle = (frame * 0.02) + (i * Math.PI * 2 / particleCount);
                    const distance = player.width/2 * (1.2 + Math.sin(frame * 0.1) * 0.2);
                    const particleX = Math.cos(angle) * distance;
                    const particleY = Math.sin(angle) * distance;
                    
                    ctx.beginPath();
                    ctx.arc(
                        particleX,
                        particleY,
                        2 + Math.sin(frame * 0.1 + i) * 1,
                        0,
                        Math.PI * 2
                    );
                    ctx.fillStyle = characterConfig.accentColor;
                    ctx.fill();
                }
            }
            
            // Draw main body with enhanced ethereal effect
            const gradientAlien = ctx.createRadialGradient(0, 0, 0, 0, 0, player.width/2);
            gradientAlien.addColorStop(0, characterConfig.color);
            gradientAlien.addColorStop(0.6, characterConfig.accentColor);
            gradientAlien.addColorStop(1, characterConfig.color);
            ctx.fillStyle = gradientAlien;
            
            // Main body with wave effect
            ctx.beginPath();
            const segments = 12;
            const waveIntensity = 0.1;
            for (let i = 0; i <= segments; i++) {
                const angle = (i / segments) * Math.PI * 2;
                const waveOffset = Math.sin(angle * 3 + frame * 0.1) * waveIntensity;
                const radius = (player.width/2 * pulseSize) * (1 + waveOffset);
                const x = Math.cos(angle) * radius;
                const y = Math.sin(angle) * radius;
                
                if (i === 0) {
                    ctx.moveTo(x, y);
                } else {
                    ctx.lineTo(x, y);
                }
            }
            ctx.closePath();
            ctx.fill();

            // Enhanced antenna system
            if (characterConfig.effects.hasAntenna) {
                const antennaCount = 3;
                for (let i = 0; i < antennaCount; i++) {
                    const baseAngle = (i - 1) * Math.PI/6;
                    const waveOffset = Math.sin(frame * 0.1 + i) * 5;
                    
                    ctx.beginPath();
                    ctx.moveTo(
                        Math.sin(baseAngle) * player.width/4,
                        -player.height/2
                    );
                    
                    // Curved antenna with energy pulse
                    const controlX = Math.sin(baseAngle) * player.width/2 + waveOffset;
                    const controlY = -player.height/2 - 15;
                    const endX = Math.sin(baseAngle) * player.width/3 + waveOffset;
                    const endY = -player.height/2 - 25;
                    
                    ctx.quadraticCurveTo(controlX, controlY, endX, endY);
                    ctx.strokeStyle = characterConfig.accentColor;
                    ctx.lineWidth = 2;
                    ctx.stroke();
                    
                    // Add glowing tips
                    const glowSize = 2 + Math.sin(frame * 0.1 + i * Math.PI/2) * 1;
                    ctx.beginPath();
                    ctx.arc(endX, endY, glowSize, 0, Math.PI * 2);
                    ctx.fillStyle = characterConfig.color;
                    ctx.fill();
                    
                    // Add energy pulse along antenna
                    const pulsePos = (Math.sin(frame * 0.05 + i) + 1) / 2;
                    const pulseX = Math.sin(baseAngle) * player.width/4 + (endX - Math.sin(baseAngle) * player.width/4) * pulsePos;
                    const pulseY = -player.height/2 + (endY - (-player.height/2)) * pulsePos;
                    
                    ctx.beginPath();
                    ctx.arc(pulseX, pulseY, 2, 0, Math.PI * 2);
                    ctx.fillStyle = characterConfig.accentColor;
                    ctx.fill();
                }
            }
            break;
        }
    }

    // Reset shadow for eyes
    ctx.shadowColor = 'transparent';
    ctx.shadowBlur = 0;
    ctx.shadowOffsetY = 0;

    // Draw enhanced eyes with animation
    function drawEnhancedEyes(x: number, y: number, size: number) {
        const blinkState = Math.sin(frame * 0.1) > 0.95 ? 0 : 1;
        
        ctx.save();
        // Eye glow effect
        ctx.shadowColor = characterConfig.eyeColor;
        ctx.shadowBlur = 5;
        
        // Draw eye shape
        ctx.fillStyle = characterConfig.eyeColor;
        ctx.beginPath();
        ctx.ellipse(x, y, size, size * blinkState, 0, 0, Math.PI * 2);
        ctx.fill();
        
        // Add eye shine
        const shineSize = size * 0.4;
        ctx.beginPath();
        ctx.arc(x - size/3, y - size/3, shineSize, 0, Math.PI * 2);
        ctx.fillStyle = 'rgba(255, 255, 255, 0.8)';
        ctx.fill();
        
        ctx.restore();
    }

    // Draw character-specific eyes with enhanced effects
    switch(characterConfig.shape) {
        case 'round': {
            drawEnhancedEyes(-player.width/4, -player.height/6, player.width/8);
            drawEnhancedEyes(player.width/4, -player.height/6, player.width/8);
            break;
        }
        case 'diamond': {
            drawEnhancedEyes(-player.width/4, 0, player.width/8);
            drawEnhancedEyes(player.width/4, 0, player.width/8);
            break;
        }
        case 'square': {
            drawEnhancedEyes(-player.width/4, -player.height/6, player.width/8);
            drawEnhancedEyes(player.width/4, -player.height/6, player.width/8);
           
            break;
        }
        case 'oval': {
            drawEnhancedEyes(-player.width/4, -player.height/6, player.width/8);
            drawEnhancedEyes(player.width/4, -player.height/6, player.width/8);
            break;
        }
    }

    ctx.restore();

    // Add character-specific particle effects
    if (player.velocityY !== 0 || player.jumping) {
        const particleCount = 2;
        for (let i = 0; i < particleCount; i++) {
            particles.push({
                x: player.x + player.width/2,
                y: player.y + player.height/2,
                size: Math.random() * 3 + 2,
                speedX: (Math.random() - 0.5) * 3,
                speedY: (Math.random() - 0.5) * 3,
                life: 1,
                color: characterConfig.effects.trailColor
            });
        }
    }

    // Add enhanced teleport visual effects for alien
    if (selectedCharacter === 'alien') {
        // Draw teleport cooldown indicator
        const cooldownProgress = Math.max(0, 1 - player.teleportCooldown / 180);
        const indicatorRadius = player.width * 0.6;
        
        ctx.save();
        ctx.translate(player.x + player.width / 2, player.y + player.height / 2);
        
        // Draw cooldown arc with enhanced visuals
        ctx.beginPath();
        ctx.arc(0, 0, indicatorRadius, 0, Math.PI * 2 * cooldownProgress);
        ctx.strokeStyle = player.isTeleporting ? '#ff69b4' : characterConfig.color;
        ctx.lineWidth = 2;
        ctx.lineCap = 'round';
        ctx.stroke();
        
        // Add pulsing effect during phase
        if (player.isTeleporting) {
            ctx.globalAlpha = 0.3 + Math.sin(frame * 0.2) * 0.2;
            ctx.shadowColor = '#ff69b4';
            ctx.shadowBlur = 20;
            ctx.beginPath();
            ctx.arc(0, 0, indicatorRadius * (1 + Math.sin(frame * 0.1) * 0.1), 0, Math.PI * 2);
            ctx.strokeStyle = '#ff1493';
            ctx.stroke();
        }
        
        ctx.restore();
    }
}

// Helper function to convert hex to rgb
function hexToRgb(hex: string) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        r: parseInt(result[1], 16),
        g: parseInt(result[2], 16),
        b: parseInt(result[3], 16)
    } : null;
}

// Update jump function
function jump() {
    if (gameOver) return;
    
    const character = characters[selectedCharacter as keyof typeof characters];
    
    // Allow jump if on ground or double jump for ninja
    if (!player.jumping || (character.shape === 'diamond' && !player.doubleJumped)) {
        // Reset parachute state on new jump
        if (!player.jumping) {
            player.isParachuteActive = false;
            player.parachuteDeployed = false;
        }
        
        // Set jump velocity
        player.velocityY = character.physics.jumpForce;
        
        // Handle double jump
        if (player.jumping && !player.doubleJumped) {
            player.doubleJumped = true;
            player.rotation = 0;
        }
        
        player.jumping = true;
        player.squash = 0.8;
    }
}

// Game over function
function handleGameOver() {
    if (!gameOverEl || !finalScoreEl) return;
    
    gameOver = true;
    
    // Update final score
    finalScoreEl.textContent = score.toString();
    
    // Store high score if current score is higher
    if (score > highScore) {
        highScore = score;
        localStorage.setItem('runnerHighScore', score.toString());
    }
    
    // Show game over screen
    gameOverEl.style.display = 'flex';
    gameOverEl.style.opacity = '1';
    
    // Clear game loop
    if (gameLoop) {
        clearInterval(gameLoop);
    }
    gameStarted = false;
}

// Initialize background elements
function initBackground() {
    // Create clouds
    for (let i = 0; i < 12; i++) {
        const cloudWidth = 80 + Math.random() * 60;
        const cloudHeight = 40 + Math.random() * 30;
        const puffs = [];
        
        // Create main puff
        puffs.push({
            x: cloudWidth / 2,
            y: cloudHeight / 2,
            radius: cloudHeight / 2
        });
        
        // Create 3-5 additional puffs for each cloud
        const numPuffs = 3 + Math.floor(Math.random() * 3);
        for (let j = 0; j < numPuffs; j++) {
            puffs.push({
                x: (j + 1) * cloudWidth / (numPuffs + 1),
                y: cloudHeight / 2 + (Math.random() * 10 - 5),
                radius: (cloudHeight / 2) * (0.6 + Math.random() * 0.4)
            });
        }

        background.clouds.push({
            x: Math.random() * canvas.width,
            y: Math.random() * (canvas.height / 3),
            width: cloudWidth,
            height: cloudHeight,
            speed: 0.3 + Math.random() * 0.3,
            puffs
        });
    }

    // Create mountains
    for (let i = 0; i < 5; i++) {
        background.mountains.push({
            x: (i * canvas.width / 4),
            height: 100 + Math.random() * 100
        });
    }

    // Create stars
    for (let i = 0; i < 100; i++) {
        background.stars.push({
            x: Math.random() * canvas.width,
            y: Math.random() * (canvas.height / 2),
            size: 1 + Math.random() * 2,
            twinkle: Math.random() * Math.PI * 2
        });
    }
}

// Draw clouds function
function drawCloud(cloud: typeof background.clouds[0]) {
    ctx.save();
    ctx.translate(cloud.x, cloud.y);

    // Create gradient for 3D effect
    const gradient = ctx.createLinearGradient(0, 0, 0, cloud.height);
    gradient.addColorStop(0, 'rgba(255, 255, 255, 0.95)');
    gradient.addColorStop(1, 'rgba(255, 255, 255, 0.8)');
    ctx.fillStyle = gradient;

    // Draw each puff
    cloud.puffs.forEach(puff => {
        ctx.beginPath();
        ctx.arc(puff.x, puff.y, puff.radius, 0, Math.PI * 2);
        ctx.fill();
    });

    // Add highlights
    ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
    cloud.puffs.forEach(puff => {
        ctx.beginPath();
        ctx.arc(puff.x - puff.radius * 0.2, 
                puff.y - puff.radius * 0.2, 
                puff.radius * 0.6, 0, Math.PI * 2);
        ctx.fill();
    });

    ctx.restore();
}

// Draw background
function drawBackground() {
    // Update time
    background.time += 0.01;

    // Draw sky gradient
    const gradient = ctx.createLinearGradient(0, 0, 0, canvas.height);
    gradient.addColorStop(0, '#1a2980');
    gradient.addColorStop(1, '#26d0ce');
    ctx.fillStyle = gradient;
    ctx.fillRect(0, 0, canvas.width, canvas.height);

    // Draw stars
    background.stars.forEach(star => {
        const twinkle = Math.sin(background.time + star.twinkle) * 0.5 + 0.5;
        ctx.fillStyle = `rgba(255, 255, 255, ${twinkle * 0.8})`;
        ctx.beginPath();
        ctx.arc(star.x, star.y, star.size, 0, Math.PI * 2);
        ctx.fill();
    });

    // Draw mountains
    background.mountains.forEach(mountain => {
        ctx.fillStyle = '#34495e';
        ctx.beginPath();
        ctx.moveTo(mountain.x, canvas.height);
        ctx.lineTo(mountain.x + 100, canvas.height - mountain.height);
        ctx.lineTo(mountain.x + 200, canvas.height);
        ctx.fill();

        // Add snow caps
        ctx.fillStyle = '#fff';
        ctx.beginPath();
        ctx.moveTo(mountain.x + 85, canvas.height - mountain.height + 15);
        ctx.lineTo(mountain.x + 100, canvas.height - mountain.height);
        ctx.lineTo(mountain.x + 115, canvas.height - mountain.height + 15);
        ctx.fill();
    });

    // Draw clouds with new function
    background.clouds.forEach(drawCloud);
}

// Update background
function updateBackground() {
    // Update clouds with parallax effect
    background.clouds.forEach(cloud => {
        cloud.x -= cloud.speed;
        if (cloud.x + cloud.width < 0) {
            cloud.x = canvas.width;
            cloud.y = Math.random() * (canvas.height / 3);
            
            // Randomize puff positions slightly when cloud reappears
            cloud.puffs.forEach(puff => {
                puff.y += Math.random() * 10 - 5;
            });
        }
    });

    // Update mountains
    background.mountains.forEach(mountain => {
        mountain.x -= gameSpeed * 0.1;
        if (mountain.x + 200 < 0) {
            mountain.x = canvas.width;
            mountain.height = 100 + Math.random() * 100;
        }
    });
}

// Update game state
function update() {
    if (gameOver || isPaused) return;
    
    frame++;
    updateGameSpeed();

    // Clear canvas
    ctx.clearRect(0, 0, canvas.width, canvas.height);

    // Draw and update background
    drawBackground();
    updateBackground();

    // Draw ground
    ctx.fillStyle = '#2d3436';
    ctx.fillRect(0, canvas.height - 50, canvas.width, 50);

    // Update and draw player
    updatePlayer();
    drawPlayer();

    // Update and draw obstacles
    obstacles.forEach((obstacle, index) => {
        obstacle.update();
        const obstacleBounds = obstacle.getBounds();
        
        // Improved collision detection with smaller hitbox
        const playerHitbox = {
            x: player.x + player.width * 0.2,
            y: player.y + player.height * 0.1,
            width: player.width * 0.6,
            height: player.height * 0.8
        };
        
        if (
            !player.isTeleporting && // Only check collisions if not teleporting
            playerHitbox.x < obstacleBounds.x + obstacleBounds.width &&
            playerHitbox.x + playerHitbox.width > obstacleBounds.x &&
            playerHitbox.y < obstacleBounds.y + obstacleBounds.height &&
            playerHitbox.y + playerHitbox.height > obstacleBounds.y
        ) {
            handleGameOver();
            return;
        }

        if (obstacleBounds.x + obstacleBounds.width < 0) {
            obstacles.splice(index, 1);
            score++;
            if (scoreEl) {
                scoreEl.textContent = score.toString();
            }
        }
    });

    // Generate new obstacles
    if (obstacles.length === 0 || obstacles[obstacles.length - 1].getBounds().x < canvas.width - 300) {
        generatePlatform();
    }

    // Update and check darts
    darts = darts.filter(dart => {
        dart.update();

        // Remove darts that are off screen
        if (dart.x > canvas.width) {
            return false;
        }

        // Check for collisions with walking enemies
        let hasHit = false;
        walkingEnemies = walkingEnemies.filter(enemy => {
            if (hasHit) return true;

            const dartBounds = dart.getBounds();
            const enemyBounds = enemy.getBounds();

            if (
                dartBounds.x < enemyBounds.x + enemyBounds.width &&
                dartBounds.x + dartBounds.width > enemyBounds.x &&
                dartBounds.y < enemyBounds.y + enemyBounds.height &&
                dartBounds.y + dartBounds.height > enemyBounds.y
            ) {
                // Create explosion particles
                for (let j = 0; j < 20; j++) {
                    const angle = (j / 20) * Math.PI * 2;
                    const speed = 2 + Math.random() * 6;
                    particles.push({
                        x: enemy.x + enemy.width / 2,
                        y: enemy.y + enemy.height / 2,
                        size: Math.random() * 6 + 3,
                        speedX: Math.cos(angle) * speed,
                        speedY: Math.sin(angle) * speed,
                        life: 1,
                        color: enemy.color,
                        rotation: Math.random() * Math.PI * 2,
                        rotationSpeed: (Math.random() - 0.5) * 0.2
                    });
                }
                
                score += 5;
                if (scoreEl) {
                    scoreEl.textContent = score.toString();
                }
                hasHit = true;
                return false; // Remove the enemy
            }
            return true;
        });

        return !hasHit; // Remove dart if it hit something
    });

    // Update laser beam and check collisions
    if (laserBeam) {
        if (!laserBeam.update()) {
            laserBeam = null;
        } else {
            // Check for collisions with walking enemies
            walkingEnemies = walkingEnemies.filter(enemy => {
                const enemyBounds = enemy.getBounds();
                const beamBounds = laserBeam!.getBounds();

                if (
                    enemyBounds.x < beamBounds.x + beamBounds.width &&
                    enemyBounds.x + enemyBounds.width > beamBounds.x &&
                    enemyBounds.y < beamBounds.y + beamBounds.height &&
                    enemyBounds.y + enemyBounds.height > beamBounds.y
                ) {
                    // Create laser impact particles
                    for (let i = 0; i < 20; i++) {
                        const angle = (i / 20) * Math.PI * 2;
                        const speed = 2 + Math.random() * 4;
                        particles.push({
                            x: enemy.x + enemy.width/2,
                            y: enemy.y + enemy.height/2,
                            size: Math.random() * 4 + 2,
                            speedX: Math.cos(angle) * speed,
                            speedY: Math.sin(angle) * speed,
                            life: 1,
                            color: '#ff69b4',
                            fadeSpeed: 0.03,
                            glow: true
                        });
                    }
                    score += 5;
                    if (scoreEl) {
                        scoreEl.textContent = score.toString();
                    }
                    return false; // Remove the enemy
                }
                return true;
            });
        }
    }

    // Update hammers and check collisions
    hammers = hammers.filter(hammer => {
        if (!hammer.hasHit) {
            hammer.update();

            // Check for collisions with walking enemies
            walkingEnemies = walkingEnemies.filter(enemy => {
                const hammerBounds = hammer.getBounds();
                const enemyBounds = enemy.getBounds();

                if (
                    hammerBounds.x < enemyBounds.x + enemyBounds.width &&
                    hammerBounds.x + hammerBounds.width > enemyBounds.x &&
                    hammerBounds.y < enemyBounds.y + enemyBounds.height &&
                    hammerBounds.y + hammerBounds.height > enemyBounds.y
                ) {
                    // Create hammer impact explosion
                    for (let j = 0; j < 20; j++) {
                        const angle = (j / 20) * Math.PI * 2;
                        const speed = 3 + Math.random() * 5;
                        particles.push({
                            x: enemy.x + enemy.width/2,
                            y: enemy.y + enemy.height/2,
                            size: Math.random() * 5 + 3,
                            speedX: Math.cos(angle) * speed,
                            speedY: Math.sin(angle) * speed,
                            life: 1,
                            color: '#808080',
                            rotation: Math.random() * Math.PI * 2,
                            rotationSpeed: (Math.random() - 0.5) * 0.2,
                            gravity: 0.2,
                            fadeSpeed: 0.02
                        });
                    }
                    
                    score += 5;
                    if (scoreEl) {
                        scoreEl.textContent = score.toString();
                    }
                    hammer.hasHit = true;
                    return false; // Remove the enemy
                }
                return true;
            });

            // Check for collisions with birds
            obstacles = obstacles.filter(obstacle => {
                if (obstacle instanceof Bird) {
                    const hammerBounds = hammer.getBounds();
                    const birdBounds = obstacle.getBounds();

                    if (
                        hammerBounds.x < birdBounds.x + birdBounds.width &&
                        hammerBounds.x + hammerBounds.width > birdBounds.x &&
                        hammerBounds.y < birdBounds.y + birdBounds.height &&
                        hammerBounds.y + hammerBounds.height > birdBounds.y
                    ) {
                        // Create feather explosion effect
                        const featherColors = ['#ffffff', '#f39c12', '#ffedd5'];
                        for (let i = 0; i < 15; i++) {
                            const angle = (i / 15) * Math.PI * 2;
                            const speed = 2 + Math.random() * 4;
                            particles.push({
                                x: obstacle.x + obstacle.width/2,
                                y: obstacle.y + obstacle.height/2,
                                size: Math.random() * 4 + 2,
                                speedX: Math.cos(angle) * speed,
                                speedY: Math.sin(angle) * speed - 2,
                                life: 1,
                                color: featherColors[Math.floor(Math.random() * featherColors.length)],
                                rotation: Math.random() * Math.PI * 2,
                                rotationSpeed: (Math.random() - 0.5) * 0.1,
                                gravity: 0.1,
                                flutter: Math.random() * Math.PI * 2
                            });
                        }

                        // Add hammer impact effect
                        for (let i = 0; i < 10; i++) {
                            const angle = (i / 10) * Math.PI * 2;
                            const speed = 3 + Math.random() * 5;
                            particles.push({
                                x: obstacle.x + obstacle.width/2,
                                y: obstacle.y + obstacle.height/2,
                                size: Math.random() * 3 + 2,
                                speedX: Math.cos(angle) * speed,
                                speedY: Math.sin(angle) * speed,
                                life: 1,
                                color: '#808080',
                                rotation: Math.random() * Math.PI * 2,
                                rotationSpeed: (Math.random() - 0.5) * 0.2,
                                gravity: 0.2,
                                fadeSpeed: 0.02
                            });
                        }

                        score += 3;
                        if (scoreEl) {
                            scoreEl.textContent = score.toString();
                        }
                        hammer.hasHit = true;
                        return false; // Remove the bird
                    }
                }
                return true;
            });
        }
        return !hammer.hasHit;
    });

    // Draw hammers
    hammers.forEach(hammer => hammer.draw());

    // Update enemy spawn cooldown
    if (enemySpawnCooldown > 0) {
        enemySpawnCooldown--;
    }

    // Update and draw walking enemies
    walkingEnemies = walkingEnemies.filter(enemy => {
        enemy.update();
        
        // Remove enemies that are off screen
        if (enemy.x + enemy.width < 0) {
            return false;
        }
        return true;
    });

    // Update and check enemy projectiles
    enemyProjectiles = enemyProjectiles.filter(projectile => {
        projectile.update();

        // Check collision with player
        const projectileBounds = projectile.getBounds();
        const playerHitbox = {
            x: player.x + player.width * 0.2,
            y: player.y + player.height * 0.1,
            width: player.width * 0.6,
            height: player.height * 0.8
        };

        if (
            !player.isTeleporting &&
            projectileBounds.x < playerHitbox.x + playerHitbox.width &&
            projectileBounds.x + projectileBounds.width > playerHitbox.x &&
            projectileBounds.y < playerHitbox.y + playerHitbox.height &&
            projectileBounds.y + projectileBounds.height > playerHitbox.y
        ) {
            handleGameOver();
            return false;
        }

        // Remove projectiles that are off screen
        return projectile.x > 0;
    });

    // Update and check slime projectiles
    slimeProjectiles = slimeProjectiles.filter(slime => {
        slime.update();

        // Check collisions with birds
        obstacles = obstacles.filter(obstacle => {
            if (obstacle instanceof Bird) {
                const slimeBounds = slime.getBounds();
                const birdBounds = obstacle.getBounds();

                if (
                    slimeBounds.x < birdBounds.x + birdBounds.width &&
                    slimeBounds.x + slimeBounds.width > birdBounds.x &&
                    slimeBounds.y < birdBounds.y + birdBounds.height &&
                    slimeBounds.y + slimeBounds.height > birdBounds.y
                ) {
                    // Create slime splatter effect
                    for (let i = 0; i < 15; i++) {
                        const angle = (i / 15) * Math.PI * 2;
                        const speed = 2 + Math.random() * 3;
                        particles.push({
                            x: obstacle.x + obstacle.width/2,
                            y: obstacle.y + obstacle.height/2,
                            size: Math.random() * 4 + 2,
                            speedX: Math.cos(angle) * speed,
                            speedY: Math.sin(angle) * speed,
                            life: 1,
                            color: '#00b894',
                            fadeSpeed: 0.02,
                            gravity: 0.1
                        });
                    }

                    // Add score
                    score += 2;
                    if (scoreEl) scoreEl.textContent = score.toString();

                    return false; // Remove the bird
                }
                return true;
            }
            return true;
        });

        return slime.x < canvas.width; // Remove if off screen
    });
}

// Initialize game
export function initGame(canvasElement: HTMLCanvasElement, container: HTMLDivElement) {
    // Initialize canvas and context
    canvas = canvasElement;
    ctx = canvas.getContext('2d')!;
    
    // Get required elements
    scoreEl = container.querySelector('#scoreEl');
    finalScoreEl = container.querySelector('#finalScore');
    gameOverEl = container.querySelector('#gameOver');
    
    // Validate required elements
    if (!scoreEl || !finalScoreEl || !gameOverEl) {
        console.error('Required game elements not found in container');
        return;
    }

    // Add high score display
    const highScoreEl = document.createElement('div');
    highScoreEl.className = 'high-score';
    highScoreEl.innerHTML = `High Score: ${highScore}`;
    container.appendChild(highScoreEl);

    // Set canvas size
    canvas.width = container.clientWidth;
    canvas.height = container.clientHeight;

    // Initialize background
    initBackground();

    // Set initial player position
    player.y = canvas.height - 90;
    player.jumping = false;
    player.doubleJumped = false;
    player.velocityY = 0;

    // Clear any existing intervals
    if (gameLoop) {
        clearInterval(gameLoop);
    }

    // Ensure game over screen is properly centered
    if (gameOverEl) {
        gameOverEl.style.display = 'none';
        gameOverEl.style.position = 'fixed';
        gameOverEl.style.top = '50%';
        gameOverEl.style.left = '50%';
        gameOverEl.style.transform = 'translate(-50%, -50%)';
        gameOverEl.style.zIndex = '100';
        gameOverEl.style.backgroundColor = 'rgba(0, 0, 0, 0.9)';
        gameOverEl.style.padding = '2rem';
        gameOverEl.style.borderRadius = '1rem';
        gameOverEl.style.color = 'white';
        gameOverEl.style.textAlign = 'center';
        gameOverEl.style.flexDirection = 'column';
        gameOverEl.style.gap = '1rem';
        gameOverEl.style.width = 'min(90%, 400px)';
        gameOverEl.style.maxWidth = '400px';
    }

    // Initialize music
    initializeMusic(container);

    // Start game loop
    gameLoop = window.setInterval(update, 1000/60);
    gameStarted = true;

    // Start music when game starts
    if (!isMusicPlaying) {
        gameMusic.play().catch(() => {
            console.log('Music autoplay prevented by browser');
        });
        isMusicPlaying = true;
        const musicControls = container.querySelector('.music-controls button');
        if (musicControls) {
            musicControls.innerHTML = '🔊';
        }
    }

    // Remove old event listeners
    window.removeEventListener('keydown', handleKeyDown);
    canvas.removeEventListener('touchstart', handleTouch);
    canvas.removeEventListener('click', handleClick);

    // Add event listeners
    window.addEventListener('keydown', handleKeyDown);
    canvas.addEventListener('touchstart', handleTouch);
    canvas.addEventListener('click', handleClick);

    // Set initial game speed
    gameSpeed = INITIAL_GAME_SPEED;

    createPauseOverlay(container);
    
    // Add CSS animations
    const style = document.createElement('style');
    style.textContent = `
        @keyframes fadeIn {
            from { opacity: 0; transform: scale(0.9); }
            to { opacity: 1; transform: scale(1); }
        }
        @keyframes fadeOut {
            from { opacity: 1; transform: scale(1); }
            to { opacity: 0; transform: scale(0.9); }
        }
    `;
    document.head.appendChild(style);
}

// Update event handlers to be more responsive
function handleKeyDown(e: KeyboardEvent) {
    if (e.code === 'Space') {
        if (!gameStarted || gameOver) return;
        
        if (selectedCharacter === 'classic' && player.jumping && !player.parachuteDeployed) {
            // Deploy parachute for classic character during jump
            player.isParachuteActive = true;
            player.parachuteDeployed = true;
            player.velocityY = Math.min(player.velocityY, 2); // Immediately reduce falling speed
            
            // Add deployment particles
            for (let i = 0; i < 10; i++) {
                particles.push({
                    x: player.x + player.width/2 + (Math.random() - 0.5) * player.width,
                    y: player.y - player.height/2,
                    size: Math.random() * 3 + 2,
                    speedX: (Math.random() - 0.5) * 2,
                    speedY: Math.random() * -1,
                    life: 1,
                    color: 'rgba(255, 255, 255, 0.8)',
                    fadeSpeed: 0.02
                });
            }
        } else {
            jump();
        }
    } else if (e.code === 'KeyH' && selectedCharacter === 'classic' && !gameOver) {
        // Throw hammer for classic character
        hammers.push(new Hammer(player.x + player.width, player.y + player.height/2 - 15));
    } else if (e.code === 'KeyB') {
        if (selectedCharacter === 'ninja') {
            darts.push(new Dart(player.x + player.width, player.y + player.height / 2));
        } else if (selectedCharacter === 'alien' && player.teleportCooldown <= 0) {
            // Activate phase
            player.isTeleporting = true;
            player.teleportDuration = 120; // 120 frames = 2 seconds
            player.teleportCooldown = 180; // 3 seconds cooldown
            player.phaseAlpha = 1;
            player.phaseDirection = -1; // Start fading out
            
            // Create phase start particles with pink colors
            for (let i = 0; i < 20; i++) {
                const angle = (i / 20) * Math.PI * 2;
                const speed = 2 + Math.random() * 4;
                player.teleportParticles.push({
                    x: player.x + player.width / 2,
                    y: player.y + player.height / 2,
                    size: Math.random() * 4 + 2,
                    speedX: Math.cos(angle) * speed,
                    speedY: Math.sin(angle) * speed,
                    life: 1,
                    color: '#ff69b4', // Hot pink for phase particles
                    fadeSpeed: 0.03,
                    glow: true
                });
            }
        } else if (selectedCharacter === 'robot' && !laserBeam) {
            // Create new laser beam from drone position
            laserBeam = new LaserBeam(
                player.x + player.width/2,
                player.y - player.height * 1.2 // Drone position
            );
        }
    } else if (e.code === 'KeyG' && selectedCharacter === 'alien' && !gameOver) {
        slimeProjectiles.push(new SlimeProjectile(
            player.x + player.width,
            player.y + player.height/2
        ));
    }

    // Add pause functionality
    if (e.code === 'KeyS') {
        togglePause();
        e.preventDefault();
    }
}

function handleTouch(e: TouchEvent) {
    e.preventDefault();
    requestAnimationFrame(jump);
}

function handleClick(e: MouseEvent) {
    e.preventDefault();
    requestAnimationFrame(jump);
}

// Update reset game function to match
export function resetGame() {
    if (!gameOverEl || !scoreEl) {
        console.error('Required game elements not found');
        return;
    }

    // Reset game state
    score = 0;
    obstacles = [];
    particles = [];
    darts = [];
    gameOver = false;
    
    // Update score display
    scoreEl.textContent = '0';
    
    // Reset player position and properties
    player.x = 100;
    player.y = canvas.height - 90;
    player.velocityY = 0;
    player.jumping = false;
    player.doubleJumped = false;
    player.rotation = 0;
    player.squash = 1;
    
    // Hide game over screen - using only one class approach
    gameOverEl.classList.add('hidden');
    
    // Clear any existing game loop
    if (gameLoop) {
        clearInterval(gameLoop);
    }
    
    // Start new game loop
    gameLoop = window.setInterval(update, 1000/60);
    gameStarted = true;

    // Restart music if it was playing
    if (isMusicPlaying) {
        gameMusic.currentTime = 0;
        gameMusic.play().catch(() => {
            console.log('Music autoplay prevented by browser');
        });
    }

    isPaused = false;
    if (pauseOverlay) {
        pauseOverlay.style.display = 'none';
    }
}

// Export necessary functions and variables
export { 
    score,
    gameSpeed,
    gameOver,
    gameStarted
};

// Add Dart class after Bird class
class Dart {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    rotation: number;
    spinSpeed: number;
    trailParticles: Array<{x: number, y: number, alpha: number}> = [];
    glowIntensity: number = 1;
    pulsePhase: number = 0;

    constructor(playerX: number, playerY: number) {
        this.x = playerX;
        this.y = playerY;
        // Reduce width and height by ~30%
        this.width = 20;  // Was previously larger
        this.height = 20; // Was previously larger
        this.speed = 12;
        this.rotation = 0;
        this.spinSpeed = 0.2;
    }

    drawStar(x: number, y: number, spikes: number, outerRadius: number, innerRadius: number) {
        let rot = Math.PI / 2 * 3;
        const cx = x;
        const cy = y;
        const step = Math.PI / spikes;
    
    ctx.beginPath();
        ctx.moveTo(cx, cy - outerRadius);

        for (let i = 0; i < spikes; i++) {
            const x1 = cx + Math.cos(rot) * outerRadius;
            const y1 = cy + Math.sin(rot) * outerRadius;
            ctx.lineTo(x1, y1);
            rot += step;

            const x2 = cx + Math.cos(rot) * innerRadius;
            const y2 = cy + Math.sin(rot) * innerRadius;
            ctx.lineTo(x2, y2);
            rot += step;
        }

        ctx.lineTo(cx, cy - outerRadius);
        ctx.closePath();
    }

    draw() {
        ctx.save();
        ctx.translate(this.x, this.y);
        ctx.rotate(this.rotation);

        // Draw ninja star trail with adjusted size
        this.trailParticles.forEach(particle => {
            const gradient = ctx.createRadialGradient(
                particle.x, particle.y, 0,
                particle.x, particle.y, 7  // Reduced from 10
            );
            gradient.addColorStop(0, `rgba(45, 52, 54, ${particle.alpha * 0.5})`);
            gradient.addColorStop(1, 'rgba(45, 52, 54, 0)');
            ctx.fillStyle = gradient;
            ctx.beginPath();
            ctx.arc(particle.x, particle.y, 7, 0, Math.PI * 2);  // Reduced from 10
            ctx.fill();
        });

        // Metallic effect for ninja star with adjusted size
        const metalGradient = ctx.createRadialGradient(0, 0, 0, 0, 0, this.width * 0.5);  // Reduced from 0.7
        metalGradient.addColorStop(0, '#ffffff');
        metalGradient.addColorStop(0.3, '#95a5a6');
        metalGradient.addColorStop(0.6, '#7f8c8d');
        metalGradient.addColorStop(1, '#2c3e50');

        // Draw main ninja star with reduced size
        ctx.fillStyle = metalGradient;
        this.drawStar(0, 0, 4, this.width * 0.5, this.width * 0.2);  // Reduced from 0.7 and 0.3
        ctx.fill();

        // Add metallic shine with adjusted size
        ctx.strokeStyle = 'rgba(255, 255, 255, 0.6)';
        ctx.lineWidth = 1.5;  // Reduced from 2
        this.drawStar(0, 0, 4, this.width * 0.5, this.width * 0.2);  // Reduced from 0.7 and 0.3
        ctx.stroke();

        // Add center detail with reduced size
        ctx.beginPath();
        ctx.arc(0, 0, this.width * 0.1, 0, Math.PI * 2);  // Reduced from 0.15
        ctx.fillStyle = '#2c3e50';
        ctx.fill();
        ctx.strokeStyle = 'rgba(255, 255, 255, 0.8)';
        ctx.lineWidth = 0.8;  // Reduced from 1
        ctx.stroke();

        // Add energy particles with adjusted size
        if (Math.random() < 0.3) {
            const angle = Math.random() * Math.PI * 2;
            const distance = Math.random() * this.width * 0.2;  // Reduced from 0.3
            particles.push({
                x: this.x + Math.cos(angle) * distance,
                y: this.y + Math.sin(angle) * distance,
                size: Math.random() * 1.5 + 0.5,  // Reduced from 2 + 1
                speedX: (Math.random() - 0.5) * 2,
                speedY: (Math.random() - 0.5) * 2,
                life: 0.5,
                color: '#95a5a6',
                fadeSpeed: 0.05,
                glow: true
            });
        }

        ctx.restore();
    }

    update() {
        // Update position
        this.x += this.speed;
        this.rotation += this.spinSpeed;

        // Update trail
        this.trailParticles.push({
            x: -this.width/2,
            y: 0,
            alpha: 1
        });

        // Update trail particles
        this.trailParticles = this.trailParticles
            .map(particle => ({
                ...particle,
                alpha: particle.alpha * 0.9
            }))
            .filter(particle => particle.alpha > 0.1);

        this.draw();
    }

    getBounds() {
        return {
            x: this.x - this.width * 0.5,
            y: this.y - this.width * 0.5,
            width: this.width,
            height: this.width
        };
    }
}

// Add darts array with existing game variables
let darts: Dart[] = []; 

// Add laser beam class after Dart class
class LaserBeam {
    x: number;
    y: number;
    width: number;
    height: number;
    duration: number;
    maxDuration: number;
    alpha: number;
    isVertical: boolean;
    transitionX: number;
    verticalHeight: number;
    animationOffset: number;
    phase: 'horizontal' | 'transition' | 'vertical';
    energyRings: { x: number; y: number; radius: number; alpha: number; }[];
    energyRipples: Array<{x: number, y: number, radius: number, alpha: number}> = [];
    plasmaField: Array<{x: number, y: number, size: number, phase: number}> = [];

    constructor(x: number, y: number) {
        this.x = x;
        this.y = y;
        this.width = 200;
        this.height = 8;
        this.duration = 300;
        this.maxDuration = 300;
        this.alpha = 1;
        this.isVertical = false;
        this.transitionX = x + 200;
        this.verticalHeight = canvas.height;
        this.animationOffset = 0;
        this.phase = 'horizontal';
        this.energyRings = [];
    }

    draw() {
        ctx.save();
        
        // Draw plasma field
        this.plasmaField.forEach(plasma => {
            const gradient = ctx.createRadialGradient(
                plasma.x, plasma.y, 0,
                plasma.x, plasma.y, plasma.size
            );
            gradient.addColorStop(0, `rgba(255, 105, 180, ${0.3 + Math.sin(plasma.phase) * 0.2})`);
            gradient.addColorStop(1, 'rgba(255, 105, 180, 0)');
            
            ctx.fillStyle = gradient;
            ctx.beginPath();
            ctx.arc(plasma.x, plasma.y, plasma.size, 0, Math.PI * 2);
            ctx.fill();
            
            plasma.phase += 0.1;
        });

        // Draw energy ripples
        this.energyRipples.forEach(ripple => {
            ctx.beginPath();
            ctx.arc(ripple.x, ripple.y, ripple.radius, 0, Math.PI * 2);
            ctx.strokeStyle = `rgba(255, 182, 193, ${ripple.alpha})`;
            ctx.lineWidth = 2;
            ctx.stroke();
            
            ripple.radius += 2;
            ripple.alpha *= 0.95;
        });
        
        // Update animation offset
        this.animationOffset += 0.2;
        
        // Draw energy core
        const coreGlow = Math.sin(this.animationOffset * 0.5) * 0.3 + 0.7;
        ctx.shadowColor = '#ff69b4';
        ctx.shadowBlur = 20 * coreGlow;
        
        if (this.phase === 'horizontal') {
            // Horizontal beam with energy pulse
            const pulseWidth = this.width * (1 + Math.sin(this.animationOffset) * 0.1);
            const pulseHeight = this.height * (1 + Math.sin(this.animationOffset * 2) * 0.1);
            
            // Draw main beam with pulse effect
            const beamGradient = ctx.createLinearGradient(this.x, this.y, this.x + pulseWidth, this.y);
            beamGradient.addColorStop(0, `rgba(255, 105, 180, ${this.alpha})`);
            beamGradient.addColorStop(0.5, `rgba(255, 255, 255, ${this.alpha * coreGlow})`);
            beamGradient.addColorStop(1, `rgba(255, 105, 180, ${this.alpha * 0.7})`);
            
            ctx.fillStyle = beamGradient;
            ctx.beginPath();
            ctx.roundRect(this.x, this.y - pulseHeight/2, pulseWidth, pulseHeight, 4);
            ctx.fill();

            // Add energy rings at beam start
            if (frame % 4 === 0) {
                this.energyRings.push({
                    x: this.x,
                    y: this.y,
                    radius: this.height,
                    alpha: 1
                });
            }
        } else if (this.phase === 'transition') {
            // Enhanced transition effect
            const progress = (this.transitionX - this.x) / 50;
            const verticalHeight = this.verticalHeight * (1 - progress);
            const horizontalWidth = this.width * progress;
            
            // Draw horizontal part with energy pulse
            const horizGradient = ctx.createLinearGradient(this.x, this.y, this.x + horizontalWidth, this.y);
            horizGradient.addColorStop(0, `rgba(255, 105, 180, ${this.alpha})`);
            horizGradient.addColorStop(1, `rgba(255, 255, 255, ${this.alpha * coreGlow})`);
            
            ctx.fillStyle = horizGradient;
            ctx.beginPath();
            ctx.roundRect(
                this.x, 
                this.y - this.height/2, 
                horizontalWidth, 
                this.height * (1 + Math.sin(this.animationOffset * 2) * 0.1),
                4
            );
            ctx.fill();

            // Draw vertical part with energy surge
            const vertGradient = ctx.createLinearGradient(this.x, 0, this.x, verticalHeight);
            vertGradient.addColorStop(0, `rgba(255, 255, 255, ${this.alpha * coreGlow})`);
            vertGradient.addColorStop(0.5, `rgba(255, 105, 180, ${this.alpha})`);
            vertGradient.addColorStop(1, `rgba(255, 105, 180, ${this.alpha * 0.7})`);
            
            ctx.fillStyle = vertGradient;
            ctx.beginPath();
            ctx.roundRect(
                this.transitionX - this.height/2,
                0,
                this.height,
                verticalHeight,
                4
            );
            ctx.fill();

            // Add transition energy burst
            if (frame % 2 === 0) {
                const burstAngle = Math.random() * Math.PI * 2;
                const burstDistance = Math.random() * 20;
                particles.push({
                    x: this.transitionX + Math.cos(burstAngle) * burstDistance,
                    y: this.y + Math.sin(burstAngle) * burstDistance,
                    size: Math.random() * 4 + 2,
                    speedX: Math.cos(burstAngle) * 3,
                    speedY: Math.sin(burstAngle) * 3,
                    life: 1,
                    color: '#ff69b4',
                    fadeSpeed: 0.05,
                    glow: true
                });
            }
        } else {
            // Enhanced vertical beam with energy waves
            const segmentHeight = 20;
            const numSegments = Math.ceil(canvas.height / segmentHeight);
            
            for (let i = 0; i < numSegments; i++) {
                const segY = i * segmentHeight;
                const waveOffset = Math.sin(i * 0.5 + this.animationOffset) * 2;
                const segmentAlpha = this.alpha * (1 + Math.sin(i * 0.2 + this.animationOffset) * 0.2);
                
                const segGradient = ctx.createLinearGradient(
                    this.x - this.height/2, segY,
                    this.x + this.height/2, segY + segmentHeight
                );
                segGradient.addColorStop(0, `rgba(255, 105, 180, ${segmentAlpha})`);
                segGradient.addColorStop(0.5, `rgba(255, 255, 255, ${segmentAlpha * coreGlow})`);
                segGradient.addColorStop(1, `rgba(255, 105, 180, ${segmentAlpha})`);
                
                ctx.fillStyle = segGradient;
                ctx.beginPath();
                ctx.roundRect(
                    this.x - this.height/2 + waveOffset, 
                    segY, 
                    this.height, 
                    segmentHeight, 
                    4
                );
                ctx.fill();

                // Add energy particles along the wave
                if (frame % 3 === 0 && Math.random() > 0.7) {
                    particles.push({
                        x: this.x + waveOffset,
                        y: segY + segmentHeight/2,
                        size: Math.random() * 3 + 1,
                        speedX: (Math.random() - 0.5) * 6,
                        speedY: (Math.random() - 0.5) * 6,
                        life: 1,
                        color: '#ff69b4',
                        fadeSpeed: 0.03,
                        glow: true
                    });
                }
            }
        }

        // Update and draw energy rings
        this.energyRings = this.energyRings.filter(ring => {
            ring.radius += 2;
            ring.alpha -= 0.05;
            
            if (ring.alpha > 0) {
                ctx.beginPath();
                ctx.arc(ring.x, ring.y, ring.radius, 0, Math.PI * 2);
                ctx.strokeStyle = `rgba(255, 105, 180, ${ring.alpha})`;
                ctx.lineWidth = 2;
                ctx.stroke();
                return true;
            }
            return false;
        });

        // Add core energy particles
        if (frame % 2 === 0) {
            const particleCount = this.phase === 'transition' ? 5 : 3;
            for (let i = 0; i < particleCount; i++) {
                const angle = Math.random() * Math.PI * 2;
                const speed = Math.random() * 4 + 2;
                const particleX = this.phase === 'vertical' ? 
                    this.x + Math.cos(angle) * this.height :
                    this.x + Math.random() * this.width;
                const particleY = this.phase === 'vertical' ? 
                    Math.random() * canvas.height :
                    this.y + Math.sin(angle) * this.height;
                
                particles.push({
                    x: particleX,
                    y: particleY,
                    size: Math.random() * 3 + 1,
                    speedX: Math.cos(angle) * speed,
                    speedY: Math.sin(angle) * speed,
                    life: 1,
                    color: Math.random() > 0.5 ? '#ff69b4' : '#ffffff',
                    fadeSpeed: 0.04,
                    glow: true
                });
            }
        }

        ctx.restore();
    }

    update() {
        this.duration--;
        this.alpha = this.duration / this.maxDuration;
        
        if (this.phase === 'horizontal') {
            this.x += 8;
            if (this.x >= this.transitionX - 50) {
                this.phase = 'transition';
            }
        } else if (this.phase === 'transition') {
            this.x += 4;
            if (this.x >= this.transitionX) {
                this.phase = 'vertical';
                this.x = this.transitionX;
            }
        }
        
        this.draw();
        return this.duration > 0;
    }

    getBounds() {
        if (this.phase === 'vertical') {
            return {
                x: this.x - this.height/2,
                y: 0,
                width: this.height,
                height: canvas.height
            };
        } else {
            return {
                x: this.x,
                y: this.y - this.height/2,
                width: this.width,
                height: this.height
            };
        }
    }
}

// Add laser beam array with game variables
let laserBeam: LaserBeam | null = null;

// Add at the top with other game variables
let gameMusic: HTMLAudioElement;
let isMusicPlaying = false;
let musicVolume = parseFloat(localStorage.getItem('runnerMusicVolume') || '0.1'); // Default 10% or saved value

// Add at the top with other game variables
let lastNonZeroVolume = parseFloat(localStorage.getItem('runnerLastVolume') || '0.1');

function initializeMusic(container: HTMLDivElement) {
    // Create audio element with error handling
    gameMusic = new Audio('/games/runner/game_music.mp3');
    gameMusic.onerror = () => {
        console.error('Error loading game music file');
        isMusicPlaying = false;
    };
    gameMusic.loop = true;
    gameMusic.volume = musicVolume;
    
    // Create more compact music control container
    const musicControls = document.createElement('div');
    musicControls.className = 'music-controls';
    musicControls.style.position = 'fixed';
    musicControls.style.top = '80px';
    musicControls.style.left = '20px';
    musicControls.style.zIndex = '1000';
    musicControls.style.display = 'flex';
    musicControls.style.alignItems = 'center';
    musicControls.style.gap = '4px';
    musicControls.style.background = 'linear-gradient(135deg, #2c3e50, #3498db)';
    musicControls.style.padding = '4px 8px';
    musicControls.style.borderRadius = '15px';
    musicControls.style.boxShadow = '0 2px 5px rgba(0,0,0,0.2)';
    musicControls.style.border = '1px solid rgba(255,255,255,0.1)';
    musicControls.style.transform = 'scale(0.9)'; // Make entire control slightly smaller

    // Create compact volume label
    const volumeLabel = document.createElement('span');
    volumeLabel.style.color = '#fff';
    volumeLabel.style.fontSize = '10px';
    volumeLabel.style.minWidth = '24px';
    volumeLabel.style.textAlign = 'right';
    volumeLabel.textContent = musicVolume === 0 ? 'Muted' : `${Math.round(musicVolume * 100)}%`;

    // Create compact play/pause button
    const toggleButton = document.createElement('button');
    toggleButton.innerHTML = musicVolume === 0 ? '🔇' : (isMusicPlaying ? '🔊' : '🔈');
    toggleButton.style.background = 'none';
    toggleButton.style.border = 'none';
    toggleButton.style.color = 'white';
    toggleButton.style.fontSize = '16px';
    toggleButton.style.cursor = 'pointer';
    toggleButton.style.padding = '2px';
    toggleButton.style.display = 'flex';
    toggleButton.style.alignItems = 'center';
    toggleButton.style.justifyContent = 'center';
    toggleButton.style.transition = 'transform 0.2s';
    toggleButton.style.width = '24px';
    toggleButton.title = 'Toggle Music';

    // Create compact volume slider
    const volumeSlider = document.createElement('input');
    volumeSlider.type = 'range';
    volumeSlider.min = '0';
    volumeSlider.max = '100';
    volumeSlider.value = (musicVolume * 100).toString();
    volumeSlider.style.width = '60px';
    volumeSlider.style.height = '3px';
    volumeSlider.style.webkitAppearance = 'none';
    volumeSlider.style.background = 'rgba(255,255,255,0.2)';
    volumeSlider.style.borderRadius = '2px';
    volumeSlider.style.cursor = 'pointer';
    volumeSlider.title = 'Volume Control';

    function updateVolumeState(newVolume: number, shouldPlay: boolean = false) {
        const wasMuted = musicVolume === 0;
        const willBeMuted = newVolume === 0;

        // Store last non-zero volume when changing to non-zero
        if (newVolume > 0) {
            lastNonZeroVolume = newVolume;
            localStorage.setItem('runnerLastVolume', lastNonZeroVolume.toString());
        }

        // Update volume and UI
        musicVolume = newVolume;
        gameMusic.volume = newVolume;
        volumeSlider.value = (newVolume * 100).toString();
        volumeLabel.textContent = willBeMuted ? 'Muted' : `${Math.round(newVolume * 100)}%`;

        // Handle muting/unmuting
        if (willBeMuted) {
            gameMusic.pause();
            isMusicPlaying = false;
            toggleButton.innerHTML = '🔇';
        } else {
            if (shouldPlay || (isMusicPlaying && wasMuted)) {
                gameMusic.play().catch(() => console.log('Playback prevented'));
                isMusicPlaying = true;
                toggleButton.innerHTML = '🔊';
            } else {
                toggleButton.innerHTML = '🔈';
            }
        }

        // Save volume preference
        localStorage.setItem('runnerMusicVolume', musicVolume.toString());
    }

    // Toggle button click handler
    toggleButton.addEventListener('click', () => {
        if (isMusicPlaying) {
            gameMusic.pause();
            isMusicPlaying = false;
            toggleButton.innerHTML = musicVolume === 0 ? '🔇' : '🔈';
        } else {
            if (musicVolume === 0) {
                // Restore last non-zero volume when unmuting
                updateVolumeState(lastNonZeroVolume, true);
            } else {
                gameMusic.play();
                isMusicPlaying = true;
                toggleButton.innerHTML = '🔊';
            }
        }
    });

    // Volume slider input handler
    volumeSlider.addEventListener('input', (e) => {
        const target = e.target as HTMLInputElement;
        const newVolume = parseInt(target.value) / 100;
        updateVolumeState(newVolume);
    });

    // Add elements to container
    musicControls.appendChild(toggleButton);
    musicControls.appendChild(volumeSlider);
    musicControls.appendChild(volumeLabel);
    container.appendChild(musicControls);

    // Initialize with saved volume
    updateVolumeState(musicVolume, true);
}

// Add hammer array with other game variables
let hammers: Hammer[] = [];

// Add Hammer class after other classes
class Hammer {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    rotation: number;
    rotationSpeed: number;
    hasHit: boolean;

    constructor(x: number, y: number) {
        this.x = x;
        this.y = y;
        this.width = 30;
        this.height = 30;
        this.speed = 12;
        this.rotation = 0;
        this.rotationSpeed = 0.4;
        this.hasHit = false;
    }

    draw() {
        ctx.save();
        ctx.translate(this.x + this.width/2, this.y + this.height/2);
        ctx.rotate(this.rotation);

        // Draw handle with gradient
        const handleGradient = ctx.createLinearGradient(
            -this.width * 0.75, 0,
            -this.width * 0.25, 0
        );
        handleGradient.addColorStop(0, '#8B4513');
        handleGradient.addColorStop(0.5, '#D2691E');
        handleGradient.addColorStop(1, '#8B4513');
        
        ctx.fillStyle = handleGradient;
        ctx.beginPath();
        ctx.roundRect(
            -this.width * 0.75, -this.height * 0.1,
            this.width * 0.5, this.height * 0.2,
            3
        );
        ctx.fill();

        // Draw hammer head with metallic effect
        const headGradient = ctx.createLinearGradient(
            -this.width * 0.25, -this.height * 0.3,
            this.width * 0.25, this.height * 0.3
        );
        headGradient.addColorStop(0, '#808080');
        headGradient.addColorStop(0.5, '#A0A0A0');
        headGradient.addColorStop(1, '#606060');

        // Main hammer head
        ctx.fillStyle = headGradient;
        ctx.beginPath();
        ctx.roundRect(
            -this.width * 0.25, -this.height * 0.3,
            this.width * 0.5, this.height * 0.6,
            5
        );
        ctx.fill();

        // Add metallic shine
        const shineGradient = ctx.createLinearGradient(
            -this.width * 0.2, -this.height * 0.25,
            this.width * 0.2, this.height * 0.25
        );
        shineGradient.addColorStop(0, 'rgba(255, 255, 255, 0)');
        shineGradient.addColorStop(0.5, 'rgba(255, 255, 255, 0.3)');
        shineGradient.addColorStop(1, 'rgba(255, 255, 255, 0)');
        
        ctx.fillStyle = shineGradient;
        ctx.beginPath();
        ctx.roundRect(
            -this.width * 0.2, -this.height * 0.25,
            this.width * 0.4, this.height * 0.5,
            4
        );
        ctx.fill();

        // Add decorative rivets
        const rivetPositions = [
            {x: -this.width * 0.2, y: 0},
            {x: this.width * 0.2, y: 0}
        ];
        
        rivetPositions.forEach(pos => {
            ctx.beginPath();
            ctx.arc(pos.x, pos.y, 2, 0, Math.PI * 2);
            ctx.fillStyle = '#404040';
            ctx.fill();
            ctx.beginPath();
            ctx.arc(pos.x, pos.y, 1, 0, Math.PI * 2);
            ctx.fillStyle = '#808080';
            ctx.fill();
        });

        ctx.restore();
    }

    update() {
        if (!this.hasHit) {
            this.x += this.speed;
            this.rotation += this.rotationSpeed;

            // Check for collisions with walking enemies
            for (let i = walkingEnemies.length - 1; i >= 0; i--) {
                const enemy = walkingEnemies[i];
                const hammerBounds = this.getBounds();
                const enemyBounds = enemy.getBounds();

                if (
                    hammerBounds.x < enemyBounds.x + enemyBounds.width &&
                    hammerBounds.x + hammerBounds.width > enemyBounds.x &&
                    hammerBounds.y < enemyBounds.y + enemyBounds.height &&
                    hammerBounds.y + hammerBounds.height > enemyBounds.y
                ) {
                    // Create hammer impact explosion
                    for (let j = 0; j < 20; j++) {
                        const angle = (j / 20) * Math.PI * 2;
                        const speed = 3 + Math.random() * 5;
                        particles.push({
                            x: enemy.x + enemy.width/2,
                            y: enemy.y + enemy.height/2,
                            size: Math.random() * 5 + 3,
                            speedX: Math.cos(angle) * speed,
                            speedY: Math.sin(angle) * speed,
                            life: 1,
                            color: '#808080',
                            rotation: Math.random() * Math.PI * 2,
                            rotationSpeed: (Math.random() - 0.5) * 0.2,
                            gravity: 0.2,
                            fadeSpeed: 0.02
                        });
                    }
                    
                    // Remove the enemy and add score
                    walkingEnemies.splice(i, 1);
                    score += 5;
                    if (scoreEl) {
                        scoreEl.textContent = score.toString();
                    }
                    this.hasHit = true;
                    return false;
                }
            }
        }
    }

    getBounds() {
        return {
            x: this.x,
            y: this.y,
            width: this.width,
            height: this.height
        };
    }
}

// Add shatter particle effect function
function createShatterEffect(x: number, y: number, color: string) {
    const particleCount = 15;
    for (let i = 0; i < particleCount; i++) {
        const angle = (Math.PI * 2 / particleCount) * i;
        const speed = 2 + Math.random() * 3;
        particles.push({
            x: x,
            y: y,
            size: Math.random() * 8 + 4,
            speedX: Math.cos(angle) * speed,
            speedY: Math.sin(angle) * speed,
            life: 1,
            color: color,
            rotation: Math.random() * Math.PI * 2,
            rotationSpeed: (Math.random() - 0.5) * 0.2,
            gravity: 0.2,
            fadeSpeed: 0.02
        });
    }
}

// Add WalkingEnemy class
class WalkingEnemy {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    color: string;
    legOffset: number;
    legDirection: number;
    shootCooldown: number;
    maxShootCooldown: number;
    armorPlates: Array<{x: number, y: number, width: number, height: number, rotation: number}> = [];
    energyCore: number = 0;
    eyePhase: number = 0;
    health: number = 2; // Add health system

    constructor() {
        this.width = 40;
        this.height = 50;
        this.x = canvas.width;
        this.y = canvas.height - 90;
        this.speed = gameSpeed * 0.4; // Slightly slower than before
        this.color = '#e74c3c';
        this.legOffset = 0;
        this.legDirection = 1;
        this.shootCooldown = 60; // Initial delay before first shot
        this.maxShootCooldown = 180; // Shoot every 3 seconds (less frequent)
        
        // Initialize armor plates
        for (let i = 0; i < 3; i++) { // Reduced number of armor plates
            this.armorPlates.push({
                x: (Math.random() - 0.5) * this.width * 0.5,
                y: (Math.random() - 0.5) * this.height * 0.5,
                width: Math.random() * 10 + 5,
                height: Math.random() * 15 + 10,
                rotation: Math.random() * Math.PI * 2
            });
        }
    }

    draw() {
        ctx.save();
        ctx.translate(this.x + this.width/2, this.y + this.height/2);

        // Draw energy aura
        const auraGradient = ctx.createRadialGradient(0, 0, 0, 0, 0, this.width);
        auraGradient.addColorStop(0, 'rgba(231, 76, 60, 0.2)');
        auraGradient.addColorStop(1, 'rgba(231, 76, 60, 0)');
        ctx.fillStyle = auraGradient;
        ctx.beginPath();
        ctx.arc(0, 0, this.width, 0, Math.PI * 2);
        ctx.fill();

        // Draw armor plates with metallic effect
        this.armorPlates.forEach(plate => {
            ctx.save();
            ctx.translate(plate.x, plate.y);
            ctx.rotate(plate.rotation);

            const plateGradient = ctx.createLinearGradient(
                -plate.width/2, -plate.height/2,
                plate.width/2, plate.height/2
            );
            plateGradient.addColorStop(0, '#c0392b');
            plateGradient.addColorStop(0.5, '#e74c3c');
            plateGradient.addColorStop(1, '#c0392b');

            ctx.fillStyle = plateGradient;
            ctx.beginPath();
            ctx.roundRect(-plate.width/2, -plate.height/2, plate.width, plate.height, 2);
            ctx.fill();

            // Add metallic shine
            ctx.strokeStyle = 'rgba(255, 255, 255, 0.3)';
            ctx.lineWidth = 1;
            ctx.stroke();

            ctx.restore();
        });

        // Draw main body with enhanced gradient
        const bodyGradient = ctx.createLinearGradient(-this.width/2, -this.height/2, this.width/2, this.height/2);
        bodyGradient.addColorStop(0, '#c0392b');
        bodyGradient.addColorStop(0.4, '#e74c3c');
        bodyGradient.addColorStop(0.6, '#e74c3c');
        bodyGradient.addColorStop(1, '#c0392b');
        
        ctx.fillStyle = bodyGradient;
        ctx.beginPath();
        ctx.roundRect(-this.width/2, -this.height/2, this.width, this.height, 8);
        ctx.fill();

        // Draw energy core
        this.energyCore += 0.1;
        const coreSize = 1 + Math.sin(this.energyCore) * 0.2;
        const coreGradient = ctx.createRadialGradient(0, 0, 0, 0, 0, 15 * coreSize);
        coreGradient.addColorStop(0, '#ffffff');
        coreGradient.addColorStop(0.5, '#e74c3c');
        coreGradient.addColorStop(1, '#c0392b');

        ctx.fillStyle = coreGradient;
        ctx.beginPath();
        ctx.arc(0, 0, 15 * coreSize, 0, Math.PI * 2);
        ctx.fill();

        // Add core glow
        ctx.shadowColor = '#e74c3c';
        ctx.shadowBlur = 15;
        ctx.fill();

        // Draw enhanced legs with animation
        this.legOffset += 0.1 * this.legDirection;
        if (Math.abs(this.legOffset) > 10) {
            this.legDirection *= -1;
        }

        [-1, 1].forEach(side => {
            // Draw leg base
            const legGradient = ctx.createLinearGradient(
                side * 10, this.height/2,
                side * 10, this.height/2 + 20
            );
            legGradient.addColorStop(0, '#c0392b');
            legGradient.addColorStop(1, '#7f8c8d');

            ctx.fillStyle = legGradient;
            ctx.beginPath();
            ctx.roundRect(
                -this.width/4 + (side * 10) - 4,
                this.height/2,
                8,
                15 + (side * this.legOffset),
                4
            );
            ctx.fill();

            // Add joint detail
            ctx.fillStyle = '#7f8c8d';
            ctx.beginPath();
            ctx.arc(
                -this.width/4 + (side * 10),
                this.height/2 + 15 + (side * this.legOffset),
                4,
                0,
                Math.PI * 2
            );
            ctx.fill();
        });

        // Draw enhanced eyes with glow
        this.eyePhase += 0.1;
        const eyeGlow = 0.7 + Math.sin(this.eyePhase) * 0.3;

        [-1, 1].forEach(side => {
            // Eye socket
            ctx.fillStyle = '#2c3e50';
            ctx.beginPath();
            ctx.arc(-this.width/4 * side, -this.height/4, 8, 0, Math.PI * 2);
            ctx.fill();

            // Glowing eye
            ctx.fillStyle = `rgba(231, 76, 60, ${eyeGlow})`;
            ctx.shadowColor = '#e74c3c';
            ctx.shadowBlur = 10;
            ctx.beginPath();
            ctx.arc(-this.width/4 * side, -this.height/4, 5, 0, Math.PI * 2);
            ctx.fill();

            // Eye highlight
            ctx.fillStyle = '#ffffff';
            ctx.shadowBlur = 0;
            ctx.beginPath();
            ctx.arc(-this.width/4 * side + 2, -this.height/4 - 2, 2, 0, Math.PI * 2);
            ctx.fill();
        });

        // Draw weapon with enhanced design
        const weaponGradient = ctx.createLinearGradient(
            this.width/4, -this.height/4,
            this.width/4 + 20, -this.height/4
        );
        weaponGradient.addColorStop(0, '#2c3e50');
        weaponGradient.addColorStop(1, '#34495e');

        ctx.fillStyle = weaponGradient;
        ctx.beginPath();
        ctx.roundRect(this.width/4, -this.height/4 - 4, 20, 8, 4);
        ctx.fill();

        // Add weapon details
        ctx.strokeStyle = '#95a5a6';
        ctx.lineWidth = 1;
        ctx.beginPath();
        ctx.moveTo(this.width/4 + 5, -this.height/4 - 4);
        ctx.lineTo(this.width/4 + 5, -this.height/4 + 4);
        ctx.stroke();
        ctx.beginPath();
        ctx.moveTo(this.width/4 + 10, -this.height/4 - 4);
        ctx.lineTo(this.width/4 + 10, -this.height/4 + 4);
        ctx.stroke();

        ctx.restore();
    }

    update() {
        this.x -= this.speed;
        this.shootCooldown--;

        if (this.shootCooldown <= 0) {
            // Create new projectile
            enemyProjectiles.push(new EnemyProjectile(
                this.x + this.width/2,
                this.y + this.height/4
            ));
            this.shootCooldown = this.maxShootCooldown;
        }

        this.draw();
    }

    getBounds() {
        return {
            x: this.x,
            y: this.y,
            width: this.width,
            height: this.height
        };
    }
}

// Add EnemyProjectile class
class EnemyProjectile {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    rotation: number;
    energyCore: number = 0;
    plasmaTrail: Array<{x: number, y: number, size: number, alpha: number}> = [];

    constructor(x: number, y: number) {
        this.x = x;
        this.y = y;
        this.width = 10;
        this.height = 4;
        this.speed = -8;
        this.rotation = 0;
    }

    draw() {
        ctx.save();
        ctx.translate(this.x, this.y);
        ctx.rotate(this.rotation);

        // Draw plasma trail
        this.plasmaTrail.forEach(plasma => {
            const gradient = ctx.createRadialGradient(
                plasma.x, plasma.y, 0,
                plasma.x, plasma.y, plasma.size
            );
            gradient.addColorStop(0, `rgba(231, 76, 60, ${plasma.alpha})`);
            gradient.addColorStop(1, 'rgba(231, 76, 60, 0)');
            
            ctx.fillStyle = gradient;
            ctx.beginPath();
            ctx.arc(plasma.x, plasma.y, plasma.size, 0, Math.PI * 2);
            ctx.fill();
        });

        // Energy core pulsing effect
        this.energyCore += 0.1;
        const coreSize = 1 + Math.sin(this.energyCore) * 0.2;

        // Enhanced projectile with energy core
        const gradient = ctx.createRadialGradient(0, 0, 0, 0, 0, this.width * 0.5);
        gradient.addColorStop(0, '#ffffff');
        gradient.addColorStop(0.5, '#e74c3c');
        gradient.addColorStop(1, '#c0392b');

        ctx.fillStyle = gradient;
        ctx.beginPath();
        ctx.roundRect(
            -this.width/2 * coreSize, 
            -this.height/2 * coreSize, 
            this.width * coreSize, 
            this.height * coreSize, 
            2
        );
        ctx.fill();

        // Add glow effect
        ctx.shadowColor = '#e74c3c';
        ctx.shadowBlur = 10 + Math.sin(this.energyCore) * 5;
        ctx.fill();

        ctx.restore();
    }

    update() {
        this.x += this.speed;
        this.rotation += 0.1;

        // Add plasma trail particles
        this.plasmaTrail.push({
            x: -this.speed,
            y: 0,
            size: Math.random() * 5 + 3,
            alpha: 1
        });

        // Update plasma trail
        this.plasmaTrail = this.plasmaTrail
            .map(plasma => ({
                ...plasma,
                alpha: plasma.alpha * 0.9,
                size: plasma.size * 0.95
            }))
            .filter(plasma => plasma.alpha > 0.1);

        this.draw();
    }

    getBounds() {
        return {
            x: this.x - this.width/2,
            y: this.y - this.height/2,
            width: this.width,
            height: this.height
        };
    }
}

// Add enemy arrays with other game variables
let walkingEnemies: WalkingEnemy[] = [];
let enemyProjectiles: EnemyProjectile[] = [];

class SlimeProjectile {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    rotation: number;
    droplets: Array<{x: number, y: number, size: number, alpha: number, speedY: number}> = [];
    wobblePhase: number = 0;
    color: string;

    constructor(x: number, y: number) {
        this.x = x;
        this.y = y;
        this.width = 15;
        this.height = 15;
        this.speed = 10;
        this.rotation = 0;
        this.color = '#00b894';  // Alien green color
    }

    draw() {
        ctx.save();
        ctx.translate(this.x, this.y);

        // Draw slime droplets
        this.droplets.forEach(droplet => {
            const dropletGradient = ctx.createRadialGradient(
                droplet.x, droplet.y, 0,
                droplet.x, droplet.y, droplet.size
            );
            dropletGradient.addColorStop(0, `rgba(0, 184, 148, ${droplet.alpha})`);
            dropletGradient.addColorStop(1, 'rgba(0, 184, 148, 0)');
            
            ctx.fillStyle = dropletGradient;
            ctx.beginPath();
            ctx.arc(droplet.x, droplet.y, droplet.size, 0, Math.PI * 2);
            ctx.fill();
        });

        // Wobble effect for main slime blob
        this.wobblePhase += 0.2;
        const wobbleX = Math.sin(this.wobblePhase) * 2;
        const wobbleY = Math.cos(this.wobblePhase * 0.5) * 2;

        // Draw main slime blob with gradient
        const gradient = ctx.createRadialGradient(0, 0, 0, wobbleX, wobbleY, this.width);
        gradient.addColorStop(0, '#00cec9');
        gradient.addColorStop(0.5, '#00b894');
        gradient.addColorStop(1, '#00a884');

        ctx.beginPath();
        ctx.moveTo(-this.width/2 + wobbleX, 0);
        for(let i = 0; i <= 20; i++) {
            const angle = (i / 20) * Math.PI * 2;
            const radius = this.width/2 + Math.sin(angle * 3 + this.wobblePhase) * 2;
            const x = Math.cos(angle) * radius + wobbleX;
            const y = Math.sin(angle) * radius + wobbleY;
            if(i === 0) ctx.moveTo(x, y);
            else ctx.lineTo(x, y);
        }
        ctx.closePath();
        ctx.fillStyle = gradient;
        ctx.fill();

        // Add glow effect
        ctx.shadowColor = '#00b894';
        ctx.shadowBlur = 10;
        ctx.fill();

        // Add shine detail
        ctx.beginPath();
        ctx.arc(-2 + wobbleX, -2 + wobbleY, 3, 0, Math.PI * 2);
        ctx.fillStyle = 'rgba(255, 255, 255, 0.7)';
        ctx.fill();

        ctx.restore();
    }

    update() {
        this.x += this.speed;

        // Add new droplets
        if(Math.random() < 0.3) {
            this.droplets.push({
                x: -this.speed + (Math.random() - 0.5) * 5,
                y: (Math.random() - 0.5) * 5,
                size: Math.random() * 3 + 2,
                alpha: 0.7,
                speedY: Math.random() * 2 - 1
            });
        }

        // Update droplets
        this.droplets = this.droplets.map(droplet => ({
            ...droplet,
            alpha: droplet.alpha * 0.95,
            size: droplet.size * 0.95,
            y: droplet.y + droplet.speedY
        })).filter(droplet => droplet.alpha > 0.1);

        this.draw();
    }

    getBounds() {
        return {
            x: this.x - this.width/2,
            y: this.y - this.height/2,
            width: this.width,
            height: this.height
        };
    }
}

// Add slimeProjectiles array with other game variables
let slimeProjectiles: SlimeProjectile[] = [];

// Add pause state
let isPaused = false;
let pauseOverlay: HTMLDivElement | null = null;

// Add pause messages for each character
const pauseMessages = {
    classic: {
        pause: "Time Out! 🕒",
        resume: "Let's Roll! 💪"
    },
    ninja: {
        pause: "Meditation Time... ⚔️",
        resume: "Back to Action! 🌟"
    },
    robot: {
        pause: "Systems Standby... 🤖",
        resume: "Reactivating Systems! ⚡"
    },
    alien: {
        pause: "Cosmic Break! 👽",
        resume: "Space Adventure Continues! 🌌"
    }
};

// Create pause overlay
function createPauseOverlay(container: HTMLDivElement) {
    pauseOverlay = document.createElement('div');
    pauseOverlay.style.position = 'absolute';
    pauseOverlay.style.top = '0';
    pauseOverlay.style.left = '0';
    pauseOverlay.style.width = '100%';
    pauseOverlay.style.height = '100%';
    pauseOverlay.style.display = 'none';
    pauseOverlay.style.justifyContent = 'center';
    pauseOverlay.style.alignItems = 'center';
    pauseOverlay.style.backgroundColor = 'rgba(0, 0, 0, 0.7)';
    pauseOverlay.style.color = '#fff';
    pauseOverlay.style.fontSize = '2em';
    pauseOverlay.style.fontWeight = 'bold';
    pauseOverlay.style.textAlign = 'center';
    pauseOverlay.style.zIndex = '1000';
    pauseOverlay.style.flexDirection = 'column';
    pauseOverlay.style.gap = '20px';
    container.appendChild(pauseOverlay);
}

// Update pause state
function togglePause() {
    if (!pauseOverlay) return;
    
    isPaused = !isPaused;
    const character = selectedCharacter as keyof typeof pauseMessages;
    const message = isPaused ? pauseMessages[character].pause : pauseMessages[character].resume;
    
    if (isPaused) {
        pauseOverlay.style.display = 'flex';
        pauseOverlay.innerHTML = `
            <div style="font-size: 3em; margin-bottom: 10px;">
                ${characters[character].effects.personality.emotes[0]}
            </div>
            <div>${message}</div>
            <div style="font-size: 0.6em; margin-top: 20px;">
                Press 'S' to ${isPaused ? 'Resume' : 'Pause'}
            </div>
        `;
        
        // Add fade-in animation
        pauseOverlay.style.animation = 'fadeIn 0.3s ease-in-out';
    } else {
        // Add fade-out animation
        pauseOverlay.style.animation = 'fadeOut 0.3s ease-in-out';
        setTimeout(() => {
            if (pauseOverlay) {
                pauseOverlay.style.display = 'none';
            }
        }, 300);
    }
}